/*
 * liballogsmat: An implementation of ALLO GSM cards
 *
 * Written by mark.liu <mark.liu@openvox.cn>
 * 
 * Modified by freedom.huang <freedom.huang@openvox.cn>
 * 
 * This program is free software, distributed under the terms of
 * the GNU General Public License Version 2 as published by the
 * Free Software Foundation. See the LICENSE file included with
 * this program for more details.
 *
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <math.h>
#include <time.h>

#include "liballogsmat.h"
#include "gsm_internal.h"
#include "gsm_module.h"
#include "gsm_config.h"

//#define TESTING 1

static char* trim_CRLF( char *String )
{
#define ISCRLF(x) ((x)=='\n'||(x)=='\r'||(x)==' ')

	char *Tail, *Head;
	for ( Tail = String + strlen( String ) - 1; Tail >= String; Tail-- ) {
		if (!ISCRLF( *Tail ))
			break;
	}
	Tail[1] = 0;

	for ( Head = String; Head <= Tail; Head ++ ) {
		if ( !ISCRLF( *Head ) )
			break;
	}

	if ( Head != String )
		memcpy( String, Head, ( Tail - Head + 2 ) * sizeof(char) );

	return String;
}

static void module_get_coverage(struct allogsm_modul *gsm, char *h)
{
	int coverage = -1;
	
	if (gsm_compare(h,get_at(gsm->switchtype,AT_CHECK_SIGNAL1))) {
		coverage = get_coverage1(gsm->switchtype,h);
	} else if (gsm_compare(h,get_at(gsm->switchtype,AT_CHECK_SIGNAL2))) {
		coverage = get_coverage2(gsm->switchtype,h);
	}
	if ((coverage) && (coverage != 99)) {
		gsm->coverage = coverage;
	} else {
		gsm->coverage = -1;
	}
}

int module_start(struct allogsm_modul *gsm) 
{
	gsm->resetting = 0;
	char tmp[5];
        sprintf(tmp, "%c", 0x1A);
        gsm_transmit(gsm, tmp);
	//gsm_switch_state(gsm, ALLOGSM_STATE_INIT, "AT+CFUN=1");
	//gsm_send_at(gsm, "AT+CFUN=1");
	gsm_switch_state(gsm, ALLOGSM_STATE_INIT, get_at(gsm->switchtype,AT_CHECK));
	//gsm_switch_state(gsm, ALLOGSM_STATE_UP, get_at(gsm->switchtype,AT_ATZ));
	return 0;
}

int module_restart(struct allogsm_modul *gsm) 
{
	gsm->resetting = 1;
	gsm_switch_state(gsm, ALLOGSM_STATE_UP, get_at(gsm->switchtype,AT_ATZ));
	return 0;
}

int module_dial(struct allogsm_modul *gsm, struct alloat_call *call) 
{
	char buf[128];

	memset(buf, 0x0, sizeof(buf));
	
	get_dial_str(gsm->switchtype, call->callednum, buf, sizeof(buf));
#ifdef WAVECOM
//	gsm_send_at(gsm, "AT+WIND=63"); /*Added for testing purpose, later commented*/
#endif
	gsm_switch_state(gsm, ALLOGSM_STATE_CALL_INIT, buf);
        gsm->dial_retry = 0;
	gsm->dial_initiated = 0;
	return 0;
}

int module_answer(struct allogsm_modul *gsm) 
{
	gsm_switch_state(gsm, ALLOGSM_STATE_PRE_ANSWER, get_at(gsm->switchtype,AT_ANSWER));
	return 0;
}

int module_senddtmf(struct allogsm_modul *gsm, int digit)
{
	char buf[512];
	snprintf(buf,512,"AT+VTS=%c",digit);
	
	return gsm_send_at(gsm, buf);
}

int module_hangup(struct allogsm_modul *gsm, struct alloat_call *c)
{
	gsm_switch_state(gsm, ALLOGSM_STATE_HANGUP_REQ, get_at(gsm->switchtype,AT_HANGUP));
        gsm->dial_initiated_hangup = 0;
        gsm->dial_initiated = 0;
	return 0;
}

static char* get_cur_time(char* buf, int size)
{
	time_t  t;
	struct tm *ptm;
	int len = 0;

	time(&t);

	ptm = localtime(&t);
	//len =  strftime(buf,size, "%Y-%m-%d %H:%M:%S", ptm);
	len =  strftime(buf,size, "%H:%M:%S", ptm);
	buf[len] = '\0';

	return buf;
}

int module_send_ussd(struct allogsm_modul *gsm, char *message) 
{	
	if (gsm->state == ALLOGSM_STATE_READY) {
//		char time_buf[20];
		char buf[1024];
/*		get_cur_time(time_buf,20);
		gsm_message(gsm, "Send USSD on span %d at %s\n",gsm->span,time_buf);*/
		snprintf(gsm->ussd_sent_buffer, sizeof(gsm->ussd_sent_buffer) - 1, "%s", message);
		get_ussd_str(gsm->switchtype, message, buf, sizeof(buf));
		gsm_switch_state(gsm, ALLOGSM_STATE_USSD_SENDING, buf);
		return 0;
	}
	
	if (gsm->debug & ALLOGSM_DEBUG_AT_RECEIVED) {
		gsm_message(gsm, "Cannot send USSD when not ready, waiting...\n");
	}
	
	return -1;
}

int module_send_text(struct allogsm_modul *gsm, char *destination, char *message) 
{	
	if (gsm->state == ALLOGSM_STATE_READY) {
		char time_buf[20];
		get_cur_time(time_buf,20);
		gsm_message(gsm, "Send SMS  to %s on span %d at %s %d \n",gsm->sms_info->txt_info.destination,gsm->span,time_buf,__LINE__);
		snprintf(gsm->sms_sent_buffer, sizeof(gsm->sms_sent_buffer) - 1, "%s", message);
//		gsm_send_at(gsm, "AT+CMMS=2"); /*Added for testing purpose, later commented*/
                gsm->sms_retry=0;
                gsm->sms_old_retry=-1;

                gsm->smsTimeoutSched = gsm_schedule_event(gsm, 30000, gsm_sms_sending_timeout, gsm);
		gsm_switch_state(gsm, ALLOGSM_STATE_SMS_SENDING, get_at(gsm->switchtype,AT_SEND_SMS_TEXT_MODE));
		return 0;
	}
	
	if (gsm->debug & ALLOGSM_DEBUG_AT_RECEIVED) {
		gsm_message(gsm, "Cannot send SMS when not ready, waiting...\n");
	}
	
	return -1;
}

int module_send_pdu( struct allogsm_modul *gsm, char *pdu)
{	
	if ((gsm->state == ALLOGSM_STATE_READY) && (gsm->dial_initiated != 1)) {
		char time_buf[20];
		get_cur_time(time_buf,20);
		gsm_message(gsm, "Send SMS to %s on span %d at %s %d\n",gsm->sms_info->txt_info.destination,gsm->span,time_buf,__LINE__);
		snprintf(gsm->sms_sent_buffer, sizeof(gsm->sms_sent_buffer) - 1, "%s", pdu);
                gsm->sms_retry=0;
                gsm->sms_old_retry=-1;

                gsm->smsTimeoutSched = gsm_schedule_event(gsm, 30000, gsm_sms_sending_timeout, gsm);		
		gsm_switch_state(gsm, ALLOGSM_STATE_SMS_SENDING, get_at(gsm->switchtype,AT_SEND_SMS_PDU_MODE));
		return 0;
	} 
	
	if (gsm->debug & ALLOGSM_DEBUG_AT_RECEIVED) {
		gsm_message(gsm, "Cannot send PDU when not ready, waiting...\n");
	}
	
	return -1;
}


int module_send_pin(struct allogsm_modul *gsm, char *pin)
{
	char buf[256];
	/* If the SIM PIN is blocked */
	if (gsm->state == ALLOGSM_STATE_SIM_PIN_REQ) {
		
		memset(buf, 0x0, sizeof(buf));
		get_pin_str(gsm->switchtype, pin, buf, sizeof(buf));
		gsm_send_at(gsm, buf);
	}

	return 0;
}


static int parse_ussd_code(struct allogsm_modul *gsm,const char* ussd_code)
{
	int response_type;
	char *temp_buffer=NULL;
	temp_buffer=strchr(ussd_code,':');
	if(temp_buffer) {
		if(strchr(temp_buffer,' ')) {
			temp_buffer=strchr(temp_buffer,' ');
			temp_buffer=temp_buffer+1;
		} else {
			temp_buffer=temp_buffer+1;
		}
	} else {
		return -1;
	}

	if(!strstr(temp_buffer,",\""))
		return -1;

	response_type=atoi(temp_buffer);

	char buf[1024];
	char *temp=buf;
	char *end=NULL;
	strncpy(buf, temp_buffer, sizeof(buf));
	memset(&gsm->ev.ussd_received,0,sizeof(gsm_event_ussd_received));
	gsm->ev.ussd_received.ussd_stat=atoi(temp);
	temp=strchr(temp,'"');
	if(!temp)
		return -1;
	else
		temp++;
	end=strrchr(temp,'"');
	if(!end)
		return -1;
	strncpy(gsm->ev.ussd_received.text,temp,end-temp);
	gsm->ev.ussd_received.text[end-temp]='\0';
	//header_len = gsm_hex2int(gsm->ev.ussd_received.text, 2) + 1;
	gsm->ev.ussd_received.len=strlen(gsm->ev.ussd_received.text);
	temp=strchr(end,',');
	if(temp)
	{
		temp++;
		gsm->ev.ussd_received.ussd_coding=atoi(temp);
	}
	else {
		gsm->ev.ussd_received.ussd_coding=0;
	}

	return response_type;
}


static allogsm_event *module_check_ussd(struct allogsm_modul *gsm, char *buf, int i)
{
	int response_type;

	if(gsm->state != ALLOGSM_STATE_USSD_SENDING)
		return NULL;

	if(gsm_compare(buf, get_at(gsm->switchtype,AT_CHECK_USSD))) {
		char *error_msg = NULL;
		response_type=parse_ussd_code(gsm,buf);
#if 0
		switch(response_type) {
		case 1:		//Successful;
			break;
		case -1:
			error_msg = "USSD parse failed\n";
		case 0:
			error_msg = "USSD response type: No further action required 0\n";
			break;
		case 2:
			error_msg = "USSD response type: USSD terminated by network 2\n";
			break;
		case 3:
			error_msg = "USSD response type: Other local client has responded 3\n";
			break;
		case 4:
			error_msg = "USSD response type: Operation not supported 4\n";
			break;
		case 5:
			error_msg = "USSD response type: Network timeout 5\n";
			break;
		default:
			error_msg = "CUSD message has unknown response type \n";
			break;
		}
#else
                if( -1 == response_type ) {
                        error_msg = "USSD parse failed\n";
                }
#endif
		if(error_msg) {
			gsm->at_last_recv[0] = '\0';
			gsm->at_pre_recv[0] = '\0';
			if (gsm->ussd_info) {
				free(gsm->ussd_info);
				gsm->ussd_info = NULL;
			}
			gsm_switch_state(gsm, ALLOGSM_STATE_READY,NULL);
			gsm->ev.e = ALLOGSM_EVENT_USSD_SEND_FAILED;
			return &gsm->ev;
        } else { //Successful
			gsm->ev.e = ALLOGSM_EVENT_USSD_RECEIVED;
			gsm->at_last_recv[0] = '\0';
			gsm->at_pre_recv[0] = '\0';
			
			if (gsm->ussd_info) {
				free(gsm->ussd_info);
				gsm->ussd_info = NULL;
			}
			gsm_switch_state(gsm, ALLOGSM_STATE_READY,NULL);
			return &gsm->ev;
		}		
	} else if (gsm_compare(buf, get_at(gsm->switchtype,AT_CMS_ERROR)) || 
			   gsm_compare(buf, get_at(gsm->switchtype,AT_NO_CARRIER)) ||
				gsm_compare(buf, get_at(gsm->switchtype,AT_CME_ERROR))) {
		gsm_error(gsm, "Error sending USSD (%s) on span %d.\n", buf, gsm->span);
		if (gsm->ussd_info) {
			free(gsm->ussd_info);
			gsm->ussd_info = NULL;
		}
		gsm_switch_state(gsm, ALLOGSM_STATE_READY,NULL);
		gsm->ev.e = ALLOGSM_EVENT_USSD_SEND_FAILED;
		return &gsm->ev;
	}

	return NULL;
}

int updateSmsStatus( struct allogsm_modul *gsm, int status){
        gsm_schedule_del(gsm, gsm->smsTimeoutSched);
        return 0;
}

static allogsm_event *module_check_sms(struct allogsm_modul *gsm, char *buf, int i)
{
	int res;
	int compare1;
	int compare2;
	char sms_buf[1024],fun[256];
        char sms_end[5];
        sprintf(sms_end, "%c", 0x1A);

	compare1 = gsm_compare(gsm->at_last_recv, get_at(gsm->switchtype,AT_CHECK_SMS));  // AT_CHECK_SMS: +CMT
	compare2 = gsm_compare(gsm->at_pre_recv, get_at(gsm->switchtype,AT_CHECK_SMS));

	if (((2 == i) && compare1) || ((1 == i) && compare2)) {
		enum sms_mode mode;
		if (2 == i) {
			mode = gsm_check_sms_mode(gsm, gsm->at_last_recv);
		} else if (1 == i) {
			mode = gsm_check_sms_mode(gsm, gsm->at_pre_recv);		
		}
		
		if (SMS_TEXT == mode) {
		    memcpy(gsm->sms_recv_buffer, gsm->at_last_recv, gsm->at_last_recv_idx);
		
			res = (2 == i) ? gsm_text2sm_event2(gsm, gsm->at_last_recv, gsm->sms_recv_buffer) : \
			                 gsm_text2sm_event2(gsm, gsm->at_pre_recv, gsm->sms_recv_buffer);
			if (!res) {
				gsm->ev.e = ALLOGSM_EVENT_SMS_RECEIVED;
				gsm->at_last_recv[0] = '\0';
				return &gsm->ev;
			} else {
				return NULL;
			}
		} else if (SMS_PDU == mode) {
			if (((2 == i) && compare1) || ((1 == i) && compare2)) {
				char *temp_buffer = NULL;
				temp_buffer = strchr(gsm->at_last_recv,',');
				if(temp_buffer) {
					temp_buffer = strstr(temp_buffer,"\r\n");
					if(temp_buffer)
						temp_buffer += 2;
					else
 						temp_buffer = gsm->sms_recv_buffer;
				} else {
					temp_buffer=gsm->sms_recv_buffer;
				}

				int len = strlen(temp_buffer);
				if((temp_buffer[len-2]=='\r')||(temp_buffer[len-2]=='\n')) {
					temp_buffer[len-2]='\0';
				} else if((temp_buffer[len-1]=='\r')||(temp_buffer[len-1]=='\n')) {
					temp_buffer[len-1]='\0';
				}

				strncpy(gsm->sms_recv_buffer, temp_buffer, sizeof(gsm->sms_recv_buffer));
				if (!gsm_pdu2sm_event(gsm, gsm->sms_recv_buffer)) {
					gsm->ev.e = ALLOGSM_EVENT_SMS_RECEIVED;
					gsm->at_last_recv[0] = '\0';
					gsm->at_pre_recv[0] = '\0';
					gsm->sms_recv_buffer[0] = '\0';
					return &gsm->ev;
				} else {
					gsm->at_pre_recv[0] = '\0';
					gsm->sms_recv_buffer[0] = '\0';
					return NULL;
				}
			}
		}
	//Freedom del 2011-12-13 17:52
	/////////////////////////////////////////////////////////////////////////////////////
#if 0
	} else if (gsm_compare(buf, get_at(gsm->switchtype,AT_ERROR)) || 
			   gsm_compare(buf, get_at(gsm->switchtype,AT_NO_CARRIER)) ||
			   gsm_compare(buf, get_at(gsm->switchtype,AT_NO_ANSWER))) {
		gsm_error(gsm, DEBUGFMT "!%s!,last at tx:[%s]\n", DEBUGARGS, buf,gsm->at_last_sent);
#endif
	/////////////////////////////////////////////////////////////////////////////////////
	}
//	printf("Buf is %s message is %s \n",buf,gsm->sms_sent_buffer);	
	switch(gsm->state) {
		case ALLOGSM_STATE_READY:
			if (gsm_compare(buf, get_at(gsm->switchtype,AT_OK))) {
				if (gsm_compare(gsm->at_last_sent, get_at(gsm->switchtype,AT_SEND_SMS_PDU_MODE))) {
			                gsm_send_at(gsm, get_at(gsm->switchtype,AT_UCS2)); /* text to pdu mode */
					gsm->sms_mod_flag = SMS_PDU;
				} else if (gsm_compare(gsm->at_last_sent, get_at(gsm->switchtype,AT_UCS2))) {
					gsm->sms_mod_flag = SMS_PDU;
				}
			}
			break;
		case ALLOGSM_STATE_SMS_SENDING:
			if (gsm_compare(buf, get_at(gsm->switchtype,AT_OK))) {
				if (gsm_compare(gsm->at_last_sent, get_at(gsm->switchtype,AT_SEND_SMS_PDU_MODE))) {
					gsm_send_at(gsm, get_at(gsm->switchtype,AT_UCS2));
				} else if (gsm_compare(gsm->at_last_sent, get_at(gsm->switchtype,AT_SEND_SMS_TEXT_MODE))) {
					gsm_send_at(gsm, get_at(gsm->switchtype,AT_GSM));
				} else if (gsm_compare(gsm->at_last_sent, get_at(gsm->switchtype,AT_UCS2))) {
					gsm->sms_mod_flag = SMS_PDU;
					memset(sms_buf, 0x0, sizeof(sms_buf));
					if (gsm->sms_info) {
						get_sms_len(gsm->switchtype, gsm->sms_info->pdu_info.len, sms_buf, sizeof(sms_buf));
					}
					gsm_send_at(gsm, sms_buf);
				} else if (gsm_compare(gsm->at_last_sent, get_at(gsm->switchtype,AT_GSM))) {
					gsm->sms_mod_flag = SMS_TEXT;
					memset(sms_buf, 0x0, sizeof(sms_buf));
					if (gsm->sms_info) {
						get_sms_des(gsm->switchtype, gsm->sms_info->txt_info.destination, sms_buf, sizeof(sms_buf));	
					}	
					gsm_send_at(gsm, sms_buf);
				} else { 
				}
			} else if (strstr(buf,">")){
                                gsm_transmit_sms(gsm, gsm->sms_sent_buffer);
//                                memset(gsm->sms_sent_buffer, 0x0, sizeof(gsm->sms_sent_buffer));
 //                               snprintf(gsm->sms_sent_buffer, sizeof(gsm->sms_sent_buffer) - 1, "%c", 0x1A);
//                                gsm_switch_state(gsm, ALLOGSM_STATE_SMS_SENT, gsm->sms_sent_buffer);
                                gsm_switch_state(gsm, ALLOGSM_STATE_SMS_SENT, sms_end);
			} else if (strstr(buf,"STIN:")||strstr(buf,"WIND:")||strstr(buf,"CREG:")){
				//gsm_switch_state(gsm, ALLOGSM_STATE_SMS_SENDING, get_at(gsm->switchtype,AT_SEND_SMS_TEXT_MODE));
				//gsm_switch_state(gsm, ALLOGSM_STATE_READY,NULL);
				if ((gsm->creg_state!=CREG_1_REGISTERED_HOME)&&(gsm->creg_state!=CREG_5_REGISTERED_ROAMING)){
					gsm_message(gsm,"Span %d: NETWORK DOWN while sending sms, network state %d\n",gsm->span, gsm->creg_state);
					gsm->sms_stuck=1;
				}else{
					if (gsm->sms_stuck){
						gsm_switch_state(gsm, ALLOGSM_STATE_SMS_SENDING, get_at(gsm->switchtype,AT_SEND_SMS_TEXT_MODE));
						gsm_message(gsm,"Span %d: NETWORK UP while sending sms, network state %d\n",gsm->span, gsm->creg_state);
					}
					gsm->sms_stuck=0;
				}
			} else if (gsm_compare(buf, get_at(gsm->switchtype,AT_CMS_ERROR)) || 
				   gsm_compare(buf, get_at(gsm->switchtype,AT_ERROR))) {
				gsm_error(gsm, "Error sending SMS (%s) on span %d.\n", buf, gsm->span);
                                updateSmsStatus(gsm, SENT_FAILED);
/*
				if (gsm->sms_info) {
					free(gsm->sms_info);
					gsm->sms_info = NULL;
				}
*/
                                if (gsm_compare(buf, "+CMS: 512")){
                                        gsm_send_at(gsm, sms_end);
                                        gsm_switch_state(gsm, ALLOGSM_STATE_INIT, get_at(gsm->switchtype,AT_GENERAL_INDICATION));
                                }else{
                                        gsm_switch_state(gsm, ALLOGSM_STATE_READY,NULL);
                                }

                                gsm->ev.e = ALLOGSM_EVENT_SMS_SEND_FAILED;
                                return &gsm->ev;

#if 0
			} else if (strstr(buf, " ")){

			} else if (expectlist_compare(buf)) {
				if (gsm_compare(gsm->at_last_sent, get_at(gsm->switchtype,AT_SEND_SMS_PDU_MODE))) {
					gsm_send_at(gsm, get_at(gsm->switchtype,AT_UCS2));
				} else if (gsm_compare(gsm->at_last_sent, get_at(gsm->switchtype,AT_SEND_SMS_TEXT_MODE))) {
					gsm_send_at(gsm, get_at(gsm->switchtype,AT_GSM));
				} else if (gsm_compare(gsm->at_last_sent, get_at(gsm->switchtype,AT_UCS2))) {
					gsm->sms_mod_flag = SMS_PDU;
					memset(sms_buf, 0x0, sizeof(sms_buf));
					if (gsm->sms_info) {
						get_sms_len(gsm->switchtype, gsm->sms_info->pdu_info.len, sms_buf, sizeof(sms_buf));
					}
					gsm_send_at(gsm, sms_buf);
				} else if (gsm_compare(gsm->at_last_sent, get_at(gsm->switchtype,AT_GSM))) {
					gsm->sms_mod_flag = SMS_TEXT;
					memset(sms_buf, 0x0, sizeof(sms_buf));
					if (gsm->sms_info) {
						get_sms_des(gsm->switchtype, gsm->sms_info->txt_info.destination, sms_buf, sizeof(sms_buf));	
					}	
					gsm_send_at(gsm, sms_buf);
				} else { 
				}
#endif
#if WAVECOM
			} else if (expectlist_compare(buf)) {
				if (strcmp(buf, " ")){
                	                gsm_send_at(gsm, sms_end );
					gsm_switch_state(gsm, ALLOGSM_STATE_SMS_SENDING, get_at(gsm->switchtype,AT_SEND_SMS_TEXT_MODE));
				}
                                if (gsm_compare(gsm->at_last_sent, get_at(gsm->switchtype,AT_SEND_SMS_PDU_MODE))) {
                                        gsm_send_at(gsm, get_at(gsm->switchtype,AT_UCS2));
                                } else if (gsm_compare(gsm->at_last_sent, get_at(gsm->switchtype,AT_SEND_SMS_TEXT_MODE))) {
                                        gsm_send_at(gsm, get_at(gsm->switchtype,AT_GSM));
                                } else if (gsm_compare(gsm->at_last_sent, get_at(gsm->switchtype,AT_UCS2))) {
                                        gsm->sms_mod_flag = SMS_PDU;
                                        memset(sms_buf, 0x0, sizeof(sms_buf));
                                        if (gsm->sms_info) {
                                                get_sms_len(gsm->switchtype, gsm->sms_info->pdu_info.len, sms_buf, sizeof(sms_buf));
                                        }
                                        gsm_send_at(gsm, sms_buf);
                                } else if (gsm_compare(gsm->at_last_sent, get_at(gsm->switchtype,AT_GSM))) {
                                        gsm->sms_mod_flag = SMS_TEXT;
                                        memset(sms_buf, 0x0, sizeof(sms_buf));
                                        if (gsm->sms_info) {
                                                get_sms_des(gsm->switchtype, gsm->sms_info->txt_info.destination, sms_buf, sizeof(sms_buf));
                                        }       
                                        gsm_send_at(gsm, sms_buf);
                                } else { 
                                        gsm_switch_state(gsm, ALLOGSM_STATE_READY, sms_end);

                                        gsm_message(gsm,"ALLO GSM: SMS Sent Failed Try again from span %d \n",gsm->span);

                                        updateSmsStatus(gsm, SENT_FAILED);

                                        gsm->ev.e = ALLOGSM_EVENT_SMS_SEND_FAILED;
                                        return &gsm->ev;
                                }

#endif
			} else {
                                if (gsm_compare(buf, "RING") || gsm_compare(buf, "+CLIP")){
                                        gsm_message(gsm,"ALLO GSM: Ring Received on %d while sending sms\n",gsm->span);
                                        updateSmsStatus(gsm, SENT_FAILED);
                                        gsm_switch_state(gsm, ALLOGSM_STATE_READY, sms_end);
                                }else{
                                        gsm_message(gsm,"Ref code at line %d. Handle this case were span is %d, buf is >>%s<<, at_last_sent >>%s<<\n", \
                                        __LINE__, gsm->span, buf, gsm->at_last_sent);
                                }
			}
			break;
		case ALLOGSM_STATE_SMS_SENT:
			if (gsm_compare(buf, get_at(gsm->switchtype,AT_SEND_SMS_SUCCESS))) {
#if 0
                                gsm_message(gsm,"ALLO GSM: SMS Sent Successfully from span %d \n",gsm->span);
                                if(gsm->sms_mod_flag==SMS_PDU)
                                {
                                        gsm_switch_state(gsm, ALLOGSM_STATE_READY,NULL);
                                        gsm->ev.e = ALLOGSM_EVENT_SMS_SEND_OK;
                                        return &gsm->ev;
                                }
                                else
                                {
                                        gsm_switch_state(gsm, ALLOGSM_STATE_SMS_SENT_END,get_at(gsm->switchtype,AT_SEND_SMS_PDU_MODE));
                                }
#else
                                gsm_message(gsm,"ALLO GSM: SMS Sent Successfully from span %d \n",gsm->span);

                                updateSmsStatus(gsm, SENT_SUCCESS);

                                gsm_switch_state(gsm, ALLOGSM_STATE_READY,NULL);
                                gsm->ev.e = ALLOGSM_EVENT_SMS_SEND_OK;
                                return &gsm->ev;
#endif
			} else if((gsm_compare(buf, get_at(gsm->switchtype,AT_CMS_ERROR)))|| (gsm_compare(buf, get_at(gsm->switchtype,AT_ERROR)))) {
                                gsm_message(gsm,"Error Received on span %d while sending sms.\n",gsm->span);
                                if (gsm->dial_initiated==1)
                                        gsm->sms_retry=4;
                                if ((gsm->sms_retry < 3) &&((gsm->sms_mod_flag == SMS_PDU) || (gsm->sms_mod_flag == SMS_TEXT))){
                                        ++(gsm->sms_retry);
                                        gsm_message(gsm,"ALLO GSM: SMS Sent Failed. Trying again from span %d \n",gsm->span);
                                        if (gsm->sms_mod_flag == SMS_PDU) {
                                                memset(sms_buf, 0x0, sizeof(sms_buf));
                                                if (gsm->sms_info) {
                                                        get_sms_len(gsm->switchtype, gsm->sms_info->pdu_info.len, sms_buf, sizeof(sms_buf));
                                                }
                                                gsm_switch_state(gsm, ALLOGSM_STATE_SMS_SENDING,sms_buf);
                                        } else if (gsm->sms_mod_flag == SMS_TEXT) {
                                                memset(sms_buf, 0x0, sizeof(sms_buf));
                                                if (gsm->sms_info) {
                                                        get_sms_des(gsm->switchtype, gsm->sms_info->txt_info.destination, sms_buf, sizeof(sms_buf));
                                                }
                                               gsm_switch_state(gsm, ALLOGSM_STATE_SMS_SENDING,sms_buf);
                                        }
                                }else{
                                        gsm_message(gsm,"ALLO GSM: SMS Sent Failed Try again from span %d \n",gsm->span);

                                        updateSmsStatus(gsm, SENT_FAILED);
                                        if (gsm_compare(buf, "+CMS ERROR: 512")){
                                                gsm_send_at(gsm, sms_end);
                                                gsm_switch_state(gsm, ALLOGSM_STATE_INIT, get_at(gsm->switchtype,AT_GENERAL_INDICATION));
                                        }else{
                                                gsm_switch_state(gsm, ALLOGSM_STATE_READY,NULL);
                                        }
                                        gsm->ev.e = ALLOGSM_EVENT_SMS_SEND_FAILED;
                                        return &gsm->ev;
                                }
			}else if(strstr(buf,"STIN:")||strstr(buf,"WIND:")||strstr(buf,"CREG:"))   {
				if ((gsm->creg_state!=CREG_1_REGISTERED_HOME)&&(gsm->creg_state!=CREG_5_REGISTERED_ROAMING)){
					gsm_message(gsm,"Span %d: NETWORK DOWN while sending sms, network state %d,  \n",gsm->span, gsm->creg_state);
					gsm->sms_stuck=1;
				}else{
					if (gsm->sms_stuck){
						gsm_switch_state(gsm, ALLOGSM_STATE_SMS_SENDING, get_at(gsm->switchtype,AT_SEND_SMS_TEXT_MODE));
						gsm_message(gsm,"Span %d: NETWORK UP while sending sms, network state %d\n",gsm->span, gsm->creg_state);
					}
					gsm->sms_stuck=0;
				}
			//	gsm_message(gsm,"SMS Sent failed on span %d\n",gsm->span);
			//	gsm_switch_state(gsm, ALLOGSM_STATE_READY,NULL);
                        //        gsm->ev.e = ALLOGSM_EVENT_SMS_SEND_FAILED;
                        //        return &gsm->ev;
			} else if (strstr(buf,">")){
                                gsm_switch_state(gsm, ALLOGSM_STATE_SMS_SENT, sms_end);
#if WAVECOM
                        } else if (expectlist_compare(buf)) {
                                if (!strcmp(buf, " ")){
					/*
                                        gsm_send_at(gsm, sms_end);
                                        gsm_switch_state(gsm, ALLOGSM_STATE_SMS_SENDING, get_at(gsm->switchtype,AT_SEND_SMS_TEXT_MODE));
					*/
                                }else{
                                        gsm_switch_state(gsm, ALLOGSM_STATE_READY, sms_end);

                                        gsm_message(gsm,"ALLO GSM: SMS Sent Failed Try again from span %d \n",gsm->span);

                                        updateSmsStatus(gsm, SENT_FAILED);

                                        gsm->ev.e = ALLOGSM_EVENT_SMS_SEND_FAILED;
                                        return &gsm->ev;
                                }
#endif
			} else {
                               if (gsm_compare(buf, "RING") || gsm_compare(buf, "+CLIP")){
                                        gsm_message(gsm,"ALLO GSM: Ring Received on %d in state SMS Sent\n",gsm->span);
                                        updateSmsStatus(gsm, SENT_FAILED);
                                        gsm_switch_state(gsm, ALLOGSM_STATE_READY,NULL);
                                }else{
                                        gsm_message(gsm,"Ref code at line %d. Handle this case were span is %d, buf is >>%s<<, at_last_sent >>%s<<\n", \
                                        __LINE__,gsm->span, buf, gsm->at_last_sent);
                                }

			}
			break;
		case ALLOGSM_STATE_SMS_SENT_END:
			if (gsm_compare(buf, get_at(gsm->switchtype,AT_OK))) {
				gsm_switch_state(gsm, ALLOGSM_STATE_READY,NULL);
				gsm->ev.e = ALLOGSM_EVENT_SMS_SEND_OK;
				return &gsm->ev;
			} else {
				gsm_switch_state(gsm, ALLOGSM_STATE_READY,NULL);
				gsm->ev.e = ALLOGSM_EVENT_SMS_SEND_FAILED;
				return &gsm->ev;
			}
			break;
		default:
			break;
	}
	return NULL;
}

static allogsm_event * module_check_network(struct allogsm_modul *gsm, struct alloat_call *call, char *buf, int i)
{
//Freedom Modify 2012-02-16 15:00
//	if (gsm->state < ALLOGSM_STATE_READY) {
//Freedom Modify 2012-06-05 15:38
//	if (gsm->state != ALLOGSM_STATE_READY) {
#ifdef WAVECOM1
	{
		char tmp1[250];
		int i=0;
		strcpy(tmp1, buf);
		trim_CRLF(tmp1);
		//if (gsm->state != ALLOGSM_STATE_NET_REQ){
		if (gsm->state < ALLOGSM_STATE_READY) {
			if (!strcmp(tmp1,get_at(gsm->switchtype,AT_CREG0))){ i=10;
			} else if (!strcmp(tmp1,get_at(gsm->switchtype,AT_CREG1))) { i=11;
			} else if (!strcmp(tmp1,get_at(gsm->switchtype,AT_CREG2))) { i=12;
			} else if (!strcmp(tmp1,get_at(gsm->switchtype,AT_CREG3))) { i=13;
			} else if (!strcmp(tmp1,get_at(gsm->switchtype,AT_CREG4))) { i=14;
			} else if (!strcmp(tmp1,get_at(gsm->switchtype,AT_CREG5))) { i=15;
			}
			if (!strcmp(tmp1,"+WIND: 0")) { i=20;
			} else if (!strcmp(tmp1,"+WIND: 1")) { i=21;
			} else if (!strcmp(tmp1,"+WIND: 2")) { i=22;
			} else if (!strcmp(tmp1,"+WIND: 3")) { i=23;
			} else if (!strcmp(tmp1,"+WIND: 4")) { i=24;
			} else if (!strcmp(tmp1,"+WIND: 5")) { i=25;
			} else if (!strcmp(tmp1,"+WIND: 6")) { i=26;
			} else if (!strcmp(tmp1,"+WIND: 7")) { i=27;
			}
		}
		if (i){	 
			char tmp[250];
			gsm_trim(gsm->at_last_sent, tmp, strlen(gsm->at_last_sent));
			printf("RETRYING Command >>%s<< because of %d:: state %s :: tmp1 %s\n", tmp, i,allogsm_state2str(gsm->state), tmp1 );
			gsm_send_at (gsm, tmp);
		}
	}
#endif
	if (gsm->state != ALLOGSM_STATE_READY && gsm->state != 	ALLOGSM_STATE_NET_REQ && gsm->state != ALLOGSM_STATE_NET_NAME_REQ) {
		return NULL;
	}
	if (gsm_compare(buf, get_at(gsm->switchtype,AT_CHECK_SIGNAL1))){
		module_get_coverage(gsm, buf);
		if (gsm->coverage < 1) {
			gsm->ev.gen.e = ALLOGSM_EVENT_NO_SIGNAL;	
			gsm_switch_state(gsm, ALLOGSM_STATE_NET_REQ, NULL);
		} else {
			gsm->ev.gen.e = ALLOGSM_EVENT_DCHAN_UP;		
			gsm_switch_state(gsm, ALLOGSM_STATE_READY, NULL);
		}
		return &gsm->ev;
	} else if (gsm_compare(buf, get_at(gsm->switchtype,AT_CREG))) {
		/*
			0 not registered, ME is not currently searching a new operator to register to
			1 registered, home network
			2 not registered, but ME is currently searching a new operator to register to
			3 registration denied
			4 unknown
			5 registered, roaming
		*/
		trim_CRLF(buf);
		if (!strcmp(buf, get_at(gsm->switchtype,AT_CREG0))) {
			gsm->network = GSM_NET_UNREGISTERED;
			gsm->ev.gen.e = ALLOGSM_EVENT_DCHAN_DOWN;
			gsm->coverage		= -1;
			gsm->net_name[0]	= 0x0;
	                gsm_switch_state(gsm, ALLOGSM_STATE_NET_REQ, get_at(gsm->switchtype,AT_ASK_NET));
			return &gsm->ev;
		} else if (!strcmp(buf, get_at(gsm->switchtype,AT_CREG1))) {
			if(gsm->state == ALLOGSM_STATE_NET_REQ) {
				gsm->network = GSM_NET_HOME;
				gsm_switch_state(gsm, ALLOGSM_STATE_NET_OK, get_at(gsm->switchtype,AT_NET_OK));
			} else {
				gsm->network = GSM_NET_HOME;
				gsm->ev.gen.e = ALLOGSM_EVENT_DCHAN_UP;
				return &gsm->ev;
			}
		} else if (!strcmp(buf, get_at(gsm->switchtype,AT_CREG2))) {
			gsm->network = GSM_NET_SEARCHING;
			gsm->ev.gen.e = ALLOGSM_EVENT_DCHAN_DOWN;
			gsm->coverage		= -1;
			gsm->net_name[0]	= 0x0;
                        gsm_switch_state(gsm, ALLOGSM_STATE_NET_REQ, get_at(gsm->switchtype,AT_ASK_NET));
			return &gsm->ev;
		} else if (!strcmp(buf, get_at(gsm->switchtype,AT_CREG3))) {
			gsm->network = GSM_NET_DENIED;
			gsm->ev.gen.e = ALLOGSM_EVENT_DCHAN_DOWN;
			gsm->coverage		= -1;
			gsm->net_name[0]	= 0x0;
                        gsm_switch_state(gsm, ALLOGSM_STATE_NET_REQ, get_at(gsm->switchtype,AT_ASK_NET));
			return &gsm->ev;
		} else if (!strcmp(buf, get_at(gsm->switchtype,AT_CREG4))) {
			gsm->network = GSM_NET_UNKNOWN;
			gsm->ev.gen.e = ALLOGSM_EVENT_DCHAN_DOWN;
			gsm->coverage		= -1;
			gsm->net_name[0]	= 0x0;
                        gsm_switch_state(gsm, ALLOGSM_STATE_NET_REQ, get_at(gsm->switchtype,AT_ASK_NET));
			return &gsm->ev;
		} else if (!strcmp(buf, get_at(gsm->switchtype,AT_CREG5))) {
			if(gsm->state == ALLOGSM_STATE_NET_REQ) {
				gsm_switch_state(gsm, ALLOGSM_STATE_NET_OK, get_at(gsm->switchtype,AT_NET_OK));
			} else {
				gsm->network = GSM_NET_ROAMING;
				gsm->ev.gen.e = ALLOGSM_EVENT_DCHAN_UP;
				return &gsm->ev;
			}
//Freedom del 2012-06-05 17:10
/*		} else {
			gsm->network		= GSM_NET_UNREGISTERED;
			gsm->coverage		= -1;
    		gsm->net_name[0]	= 0x0;
*/
		}
	} else if (gsm_compare(buf, get_at(gsm->switchtype,AT_OK))) {
		if (((2 == i) && (gsm_compare(gsm->at_last_recv, get_at(gsm->switchtype,AT_CHECK_SIGNAL1))))
			  || ((1 == i) && (gsm_compare(gsm->at_pre_recv, get_at(gsm->switchtype,AT_CHECK_SIGNAL1))))) {
			/*	
				i == 2: [ \r\n+CSQ: 25,6\r\n\r\nOK\r\n ]
				i == 1: [ \r\n+CSQ: 25,6\r\n ]
						[ \r\nOK\r\n ]
			*/
//			gsm_transmit(gsm, "AT+CREG?\r\n"); /* Req Net Status */
			if (gsm->coverage < 1) {
				gsm->ev.gen.e = ALLOGSM_EVENT_DCHAN_DOWN;
			} else {
				gsm->ev.gen.e = ALLOGSM_EVENT_DCHAN_UP;			
			}
			return &gsm->ev;
		} else if (((2 == i) && gsm_compare(gsm->at_last_recv, get_at(gsm->switchtype,AT_CREG)))
		      || ((1 == i) && gsm_compare(gsm->at_pre_recv, get_at(gsm->switchtype,AT_CREG)))) {
			/*if (GSM_NET_UNREGISTERED == gsm->network) {
				gsm->ev.gen.e = ALLOGSM_EVENT_DCHAN_DOWN;
				gsm->coverage		= -1;
				gsm->net_name[0]	= 0x0;
				return &gsm->ev;
			} else if (GSM_NET_HOME == gsm->network || GSM_NET_ROAMING == gsm->network) {
				gsm->ev.gen.e = ALLOGSM_EVENT_DCHAN_UP;
				return &gsm->ev;	
			}*/
		}
	} else if (gsm_compare(buf, get_at(gsm->switchtype,AT_CHECK_SIGNAL2))) {
		module_get_coverage(gsm, buf);
//		gsm_transmit(gsm, "AT+CREG?\r\n"); /* Req Net Status */
		if (gsm->coverage < 1) {
			gsm->ev.gen.e = ALLOGSM_EVENT_NO_SIGNAL;	
		} else {
			gsm->ev.gen.e = ALLOGSM_EVENT_DCHAN_UP;		
		}
		return &gsm->ev;
//Freedom Add 2012-06-05 15:21 adjust network state
//////////////////////////////////////////////////////////////////////////////////////////////////////
	} else if (((2 == i) && gsm_compare(gsm->at_last_recv, get_at(gsm->switchtype,AT_CREG)))
		      || ((1 == i) && gsm_compare(gsm->at_pre_recv, get_at(gsm->switchtype,AT_CREG)))) {
			/*if (GSM_NET_UNREGISTERED == gsm->network) {
				gsm->ev.gen.e = ALLOGSM_EVENT_DCHAN_DOWN;
				gsm->coverage		= -1;
				gsm->net_name[0]	= 0x0;
				return &gsm->ev;
			} else if (GSM_NET_HOME == gsm->network || GSM_NET_ROAMING == gsm->network) {
				
				gsm->ev.gen.e = ALLOGSM_EVENT_DCHAN_UP;
				return &gsm->ev;	
			}*/
	}
//////////////////////////////////////////////////////////////////////////////////////////////////////

return NULL;
}

#ifdef VIRTUAL_TTY
int module_mux_end(struct allogsm_modul *gsm, int restart_at_flow)
{
	if(restart_at_flow) {
		//Restart AT command flow again, Because after execute "AT+CMUX=0" or clear MUX mode, something need reinitialize.
		gsm->already_set_mux_mode = 1;
		return gsm_switch_state(gsm, ALLOGSM_STATE_UP, get_at(gsm->switchtype,AT_ATZ));
	} else {
		return gsm_switch_state(gsm, ALLOGSM_STATE_READY, get_at(gsm->switchtype,AT_NET_NAME));
	}
}
#endif //VIRTUAL_TTY

allogsm_event *module_receive(struct allogsm_modul *gsm, char *data, int len)
{
	struct alloat_call *call;
	char buf[1024];
	char receivebuf[1024];
	char *p;
	allogsm_event* res_event=NULL;
	int i;

	//Freedom Add 2012-02-07 15:24
	/////////////////////////////////////////////////////////
#if SIMCOM900D_NO_ANSWER_BUG
	static int first = 1;
	static struct timeval start_time,end_time;
#endif //SIMCOM900D_NO_ANSWER_BUG
	//////////////////////////////////////////////////////////

	/* get ast_call */
	call = allogsm_getcall(gsm, gsm->cref, 0);
	if (!call) {
		gsm_error(gsm, "Unable to locate call %d\n", gsm->cref);
		return NULL;
	}

	strncpy(receivebuf, data, sizeof(receivebuf));
#ifdef SUJ
	printf("RecData %s\n",receivebuf);
#endif
	p = receivebuf;
	i = 0;

	while (1) {	
		len = gsm_san(gsm, p, buf, len);
		if (0 == len || -1 == len) {
			return NULL;
		}

		if (gsm->debug & ALLOGSM_DEBUG_AT_RECEIVED)
		{
                        char tmp[1024];
                        gsm_trim(gsm->at_last_sent, tmp, strlen(gsm->at_last_sent));
                        /*
                        char tmp1[1024];
                        gsm_trim(gsm->at_last_recv, tmp1, strlen(gsm->at_last_recv));
                        char tmp2[1024];
                        gsm_trim(gsm->at_pre_recv, tmp2, strlen(gsm->at_pre_recv));
                        */
                        if (-3 == len) {
                                gsm_message(gsm, "\t\t%d<-- %d %s -- %s , NULL (----len -3----)\n", gsm->span, i, tmp, buf);
                                //gsm_message(gsm, "\t\t%d:<<< %d %s -- %s , NULL:: last_receive: %s prerecv: %s\n", gsm->span, i, tmp, buf, tmp1, tmp2);
                        }
#if 0
#define FORMAT  "\t%-2d<-- %-1d %-50s sz: %-3d || %-30.30s || last sent: %s\n"
                        gsm_message(gsm, FORMAT, gsm->span, i, buf, len, allogsm_state2str(gsm->state)+8,tmp);
#undef FORMAT
#else
#define FORMAT  "     %-2d<-- %-1d %-40.40s  || sz: %-3d|| %-23s || last sent: %s\n"
                        int ii=0;
                        gsm_message(gsm, FORMAT, gsm->span, i, buf, len, allogsm_state2str(gsm->state)+14,tmp);
                        for(ii=40; ii<len; ii=ii+40)
                                gsm_message(gsm,"             %-40.40s\n", buf+ii);
#undef FORMAT
#endif

		}

		strncpy(p, gsm->sanbuf, sizeof(receivebuf));
		len = (-3 == len) ? 0: gsm->sanidx;
		gsm->sanidx = 0;

                if(gsm_compare(buf, "+CME ERROR: 515")) {
                        gsm->CME_515_count++;
                }

                res_event = module_check_wind(gsm, call ,buf, i);
                if (res_event) {
                        return res_event;
                }
#ifdef WAVECOM
		if (gsm_compare(buf, "+CREG: ")) {
			int creg_state;
			if (!strcmp(buf,get_at(gsm->switchtype,AT_CREG0))){ creg_state = CREG_0_NOT_REG_NOT_SERARCHING;
			} else if (!strcmp(buf,get_at(gsm->switchtype,AT_CREG1))) { creg_state =CREG_1_REGISTERED_HOME;
			} else if (!strcmp(buf,get_at(gsm->switchtype,AT_CREG2))) { creg_state =CREG_2_NOT_REG_SERARCHING;
			} else if (!strcmp(buf,get_at(gsm->switchtype,AT_CREG3))) { creg_state =CREG_3_REGISTRATION_DENIED;
			} else if (!strcmp(buf,get_at(gsm->switchtype,AT_CREG4))) { creg_state =CREG_4_UNKNOWN;
			} else if (!strcmp(buf,get_at(gsm->switchtype,AT_CREG5))) { creg_state =CREG_5_REGISTERED_ROAMING;
			}
			if (!strcmp(buf,get_at(gsm->switchtype,AT_CREG10))){ creg_state = CREG_0_NOT_REG_NOT_SERARCHING;
			} else if (!strcmp(buf,get_at(gsm->switchtype,AT_CREG11))) { creg_state =CREG_1_REGISTERED_HOME;
			} else if (!strcmp(buf,get_at(gsm->switchtype,AT_CREG12))) { creg_state =CREG_2_NOT_REG_SERARCHING;
			} else if (!strcmp(buf,get_at(gsm->switchtype,AT_CREG13))) { creg_state =CREG_3_REGISTRATION_DENIED;
			} else if (!strcmp(buf,get_at(gsm->switchtype,AT_CREG14))) { creg_state =CREG_4_UNKNOWN;
			} else if (!strcmp(buf,get_at(gsm->switchtype,AT_CREG15))) { creg_state =CREG_5_REGISTERED_ROAMING;
			}
			gsm->creg_state=creg_state;
		}
#endif	
		switch (gsm->state) {
			case ALLOGSM_STATE_MANUFACTURER_REQ:
				/* Request manufacturer identification */
				if (gsm_compare(gsm->at_last_sent, get_at(gsm->switchtype,AT_GET_MANUFACTURER))) {
					if (gsm_compare(buf,get_at(gsm->switchtype,AT_OK))&&strlen(gsm->manufacturer)) {
                                                memset(gsm->revision, 0, sizeof(gsm->revision));
						gsm_switch_state(gsm, ALLOGSM_STATE_VERSION_REQ, get_at(gsm->switchtype,AT_GET_VERSION));
#ifdef WAVECOM_JUNK
					} else if (gsm_compare(buf, "+CREG: ") || gsm_compare(buf, "+WIND: ")) {
						gsm_send_at(gsm, get_at(gsm->switchtype,AT_GET_MANUFACTURER));
#endif
					} else if (!gsm_compare(buf, get_at(gsm->switchtype,AT_CME_ERROR))) {
						gsm_get_manufacturer(gsm, buf);
					} else {
						//Freedom del 2012-06-05 15:50
						//gsm_error(gsm, DEBUGFMT "!%s!,last at tx:[%s]\n", DEBUGARGS, buf,gsm->at_last_sent);
					}
				} else {
					//Freedom del 2012-06-05 15:50
					//gsm_error(gsm, DEBUGFMT "!%s!,last at tx:[%s]\n", DEBUGARGS, buf,gsm->at_last_sent);
				}
				break;
		
			case ALLOGSM_STATE_VERSION_REQ:
				if (gsm_compare(gsm->at_last_sent, get_at(gsm->switchtype,AT_GET_VERSION))) {
					if (gsm_compare(buf, get_at(gsm->switchtype,AT_OK))&&strlen(gsm->revision)) {
						gsm_switch_state(gsm, ALLOGSM_STATE_IMEI_REQ, get_at(gsm->switchtype,AT_GET_IMEI));
#ifdef WAVECOM_JUNK
					} else if (gsm_compare(buf, "+CREG: ") || gsm_compare(buf, "+WIND: ")) {
						gsm_send_at(gsm, get_at(gsm->switchtype,AT_GET_VERSION));
#endif
					} else if (!gsm_compare(buf, get_at(gsm->switchtype,AT_CME_ERROR))) {
						gsm_get_model_version(gsm, buf+9);
					} else {
						//Freedom del 2012-06-05 15:50
						//gsm_error(gsm, DEBUGFMT "!%s!,last at tx:[%s]\n", DEBUGARGS, buf,gsm->at_last_sent);
					}
				} else {
					//Freedom del 2012-06-05 15:50
					//gsm_error(gsm, DEBUGFMT "!%s!,last at tx:[%s]\n", DEBUGARGS, buf,gsm->at_last_sent);
				}
				break;
			case ALLOGSM_STATE_IMEI_REQ:
				/* IMEI (International Mobile Equipment Identification). */
				if (gsm_compare(gsm->at_last_sent, get_at(gsm->switchtype,AT_GET_IMEI))) {
					if (gsm_compare(buf, get_at(gsm->switchtype,AT_OK))&&strlen(gsm->imei)) {
						gsm_switch_state(gsm, ALLOGSM_STATE_SIM_READY_REQ, get_at(gsm->switchtype,AT_ASK_PIN));
#ifdef WAVECOM_JUNK
					} else if (gsm_compare(buf, "+CREG: ") || gsm_compare(buf, "+WIND: ")) {
						gsm_send_at(gsm, get_at(gsm->switchtype,AT_GET_IMEI));
#endif
					} else if (!gsm_compare(buf, get_at(gsm->switchtype,AT_CME_ERROR))) {
						gsm_get_imei(gsm, buf);
					} else {
						//Freedom del 2012-06-05 15:50
						//gsm_error(gsm, DEBUGFMT "!%s!,last at tx:[%s]\n", DEBUGARGS, buf,gsm->at_last_sent);
					}
				} else {
					//Freedom del 2012-06-05 15:50
					//gsm_error(gsm, DEBUGFMT "!%s!,last at tx:[%s]\n", DEBUGARGS, buf,gsm->at_last_sent);
				}
				break;
			case ALLOGSM_STATE_SIM_READY_REQ:
				if (gsm_compare(buf, get_at(gsm->switchtype,AT_PIN_READY))) {
					gsm_switch_sim_state(gsm, ALLOGSM_STATE_SIM_READY, NULL);
					allogsm_test_atcommand(gsm,"AT");
				} else if (gsm_compare(buf, get_at(gsm->switchtype,AT_PIN_SIM))) { /* waiting for SIM PIN */
					gsm_switch_sim_state(gsm, ALLOGSM_STATE_SIM_PIN_REQ, NULL);
                                        gsm_switch_state(gsm, ALLOGSM_STATE_SIM_PIN_REQ, NULL);
                                        gsm->ev.e = ALLOGSM_EVENT_PIN_REQUIRED;
                                        return &gsm->ev;
				} else if (gsm_compare(buf, get_at(gsm->switchtype,AT_OK))) {
					switch(gsm->sim_state) {
						case ALLOGSM_STATE_SIM_READY:
							gsm_switch_state(gsm, ALLOGSM_STATE_IMSI_REQ, get_at(gsm->switchtype,AT_IMSI));
							break;
						case ALLOGSM_STATE_SIM_PIN_REQ:
							gsm_switch_state(gsm, ALLOGSM_STATE_SIM_PIN_REQ, NULL);
							gsm->ev.e = ALLOGSM_EVENT_PIN_REQUIRED;
							return &gsm->ev;
							break;
					}/*+CME ERROR:incorrect passwo!*/
				}  if(gsm_compare(buf, get_at(gsm->switchtype,AT_CME_ERROR))) {
					/* Todo: other code support */
					//Freedom del 2012-06-05 15:50
					//gsm_error(gsm, DEBUGFMT "!%s!,last at tx:[%s]\n", DEBUGARGS, buf,gsm->at_last_sent);
					gsm->retries=-1;
					gsm->ev.e = ALLOGSM_EVENT_SIM_FAILED;
					return &gsm->ev;
#ifdef WAVECOM_JUNK
				} else if (gsm_compare(buf, "+CREG: ") || gsm_compare(buf, "+WIND: ") || expectlist_compare(buf)) {
					gsm_send_at(gsm, get_at(gsm->switchtype,AT_ASK_PIN));
#endif
				} else {
					//Freedom del 2012-06-05 15:50
					//gsm_error(gsm, DEBUGFMT "!%s!,last at tx:[%s]\n", DEBUGARGS, buf,gsm->at_last_sent);
				}
				break;
			case ALLOGSM_STATE_SIM_PIN_REQ:
				//if (gsm_compare(buf, get_at(gsm->switchtype,AT_OK))) { /* \r\n+CPIN: SIM PIN\r\n\r\nOK\r\n */
					/* gsm_message(gsm, "LAST>%s<\n",gsm->lastcmd); */
					allogsm_test_atcommand(gsm,"AT");
					gsm_switch_state(gsm, ALLOGSM_STATE_IMSI_REQ, get_at(gsm->switchtype,AT_IMSI));
			//	} else 
				if (gsm_compare(buf, get_at(gsm->switchtype,AT_SIM_NO_INSERTED))) {

					//Freedom Del 2011-12-13 18:47
					//////////////////////////////////////////////////////////
#if 0
					/* re-enter SIM if neccessary */
					gsm->state = ALLOGSM_STATE_SIM_PIN_REQ;
					gsm->ev.e = ALLOGSM_EVENT_PIN_REQUIRED;
					return &gsm->ev;
#endif
					//////////////////////////////////////////////////////////
					gsm->ev.e = ALLOGSM_EVENT_SIM_FAILED;
					return &gsm->ev;
				} else if(gsm_compare(buf, get_at(gsm->switchtype,AT_CME_ERROR))){
					//Freedom del 2012-06-05 15:50
					//gsm_error(gsm, DEBUGFMT "!%s!,last at tx:[%s]\n", DEBUGARGS, buf,gsm->at_last_sent);
					gsm->retries=-1;
					gsm->ev.e = ALLOGSM_EVENT_PIN_ERROR;
					return &gsm->ev;
#ifdef WAVECOM_JUNK
				} else if (gsm_compare(buf, "+CREG: ") || gsm_compare(buf, "+WIND: ") || expectlist_compare(buf)) {
					gsm_switch_state(gsm, ALLOGSM_STATE_SIM_READY_REQ, get_at(gsm->switchtype,AT_ASK_PIN));
#endif
				} else {
					//Freedom del 2012-06-05 15:50
					//gsm_error(gsm, DEBUGFMT "!%s!,last at tx:[%s]\n", DEBUGARGS, buf,gsm->at_last_sent);
				}
				break;
			case ALLOGSM_STATE_IMSI_REQ:
				/* IMSI (International Mobile Subscriber Identity number). */
				if (gsm_compare(gsm->at_last_sent, get_at(gsm->switchtype,AT_IMSI))) {
					if (gsm_compare(buf, get_at(gsm->switchtype,AT_OK))) {
						if(get_at(gsm->switchtype,AT_MOC_ENABLED))
						{
							gsm_switch_state(gsm, ALLOGSM_STATE_MOC_STATE_ENABLED, get_at(gsm->switchtype,AT_MOC_ENABLED));
						}
						else if(get_at(gsm->switchtype,AT_SET_SIDE_TONE))
						{
							gsm_switch_state(gsm, ALLOGSM_STATE_SET_SPEAKER, get_at(gsm->switchtype,AT_SPEAKER));
						}
						else
						{
							gsm_switch_state(gsm, ALLOGSM_STATE_CLIP_ENABLED, get_at(gsm->switchtype,AT_CLIP_ENABLED));
						}
					} else
					 if (!gsm_compare(buf, get_at(gsm->switchtype,AT_CME_ERROR))) {
						gsm_get_imsi(gsm, buf);
					} else {
						//Freedom del 2012-06-05 15:50
						//gsm_error(gsm, DEBUGFMT "!%s!,last at tx:[%s]\n", DEBUGARGS, buf,gsm->at_last_sent);
					}
				} else {
					//Freedom del 2012-06-05 15:50
					//gsm_error(gsm, DEBUGFMT "!%s!,last at tx:[%s]\n", DEBUGARGS, buf,gsm->at_last_sent);
				}
				break; 
			case ALLOGSM_STATE_MOC_STATE_ENABLED:
#ifdef TESTING
				if(1){
#else
				if (gsm_compare(buf, get_at(gsm->switchtype,AT_OK))) {
#endif
					if (gsm_compare(gsm->at_last_sent, get_at(gsm->switchtype,AT_MOC_ENABLED))) {
						if(get_at(gsm->switchtype,AT_SET_SIDE_TONE))
						{
							gsm_switch_state(gsm, ALLOGSM_STATE_SET_SPEAKER, get_at(gsm->switchtype,AT_SPEAKER));
						}
						else
						{
							gsm_switch_state(gsm, ALLOGSM_STATE_CLIP_ENABLED, get_at(gsm->switchtype,AT_CLIP_ENABLED));
						}
					}
#ifdef WAVECOM_JUNK
				} else if (gsm_compare(buf, "+CREG: ") || gsm_compare(buf, "+WIND: ") || expectlist_compare(buf)) {
					gsm_send_at(gsm, get_at(gsm->switchtype,AT_MOC_ENABLED));
#endif
				} else {
					//Freedom del 2012-06-05 15:50
					//gsm_error(gsm, DEBUGFMT "!%s!,last at tx:[%s]\n", DEBUGARGS, buf,gsm->at_last_sent);
				}
				break;
                        case ALLOGSM_STATE_SET_SPEAKER:

                                if (gsm_compare(buf,get_at(gsm->switchtype,AT_OK))) {
                                        if (gsm_compare(gsm->at_last_sent, get_at(gsm->switchtype,AT_SPEAKER))) {
						gsm_switch_state(gsm, ALLOGSM_STATE_SET_SIDE_TONE, get_at(gsm->switchtype,AT_SET_SIDE_TONE));
                                        }
				}
                                break;

			case ALLOGSM_STATE_SET_SIDE_TONE:
#ifdef TESTING
				if(1){
#else
				if (gsm_compare(buf, get_at(gsm->switchtype,AT_OK))) {
#endif
					if (gsm_compare(gsm->at_last_sent, get_at(gsm->switchtype,AT_SET_SIDE_TONE))) {
						gsm_switch_state(gsm, ALLOGSM_STATE_SET_SPEEK_VOL, get_at(gsm->switchtype,AT_CLVL));
					}
#ifdef WAVECOM_JUNK
				} else if (gsm_compare(buf, "+CREG: ") || gsm_compare(buf, "+WIND: ") || expectlist_compare(buf)) {
					gsm_send_at(gsm, get_at(gsm->switchtype,AT_SET_SIDE_TONE));
#endif
				} else {
					//Freedom del 2012-06-05 15:50
					//gsm_error(gsm, DEBUGFMT "!%s!,last at tx:[%s]\n", DEBUGARGS, buf,gsm->at_last_sent);
				}
				break;

			case ALLOGSM_STATE_SET_SPEEK_VOL:
#ifdef TESTING
				if(1){
#else
				if (gsm_compare(buf,get_at(gsm->switchtype,AT_OK))) {
#endif
					if (gsm_compare(gsm->at_last_sent, get_at(gsm->switchtype,AT_CLVL))) {
						gsm_switch_state(gsm, ALLOGSM_STATE_SET_MIC_VOL, get_at(gsm->switchtype,AT_CMIC));
					}
#ifdef WAVECOM_JUNK
				} else if (gsm_compare(buf, "+CREG: ") || gsm_compare(buf, "+WIND: ") || expectlist_compare(buf)) {
					gsm_send_at(gsm, get_at(gsm->switchtype, AT_CLVL));
#endif
				} else {
					//Freedom del 2012-06-05 15:50
					//gsm_error(gsm, DEBUGFMT "!%s!,last at tx:[%s]\n", DEBUGARGS, buf,gsm->at_last_sent);
				}
				break;
			case ALLOGSM_STATE_SET_MIC_VOL:
#ifdef TESTING
				if(1){
#else
				if (gsm_compare(buf,get_at(gsm->switchtype,AT_OK))) {
#endif
					if (gsm_compare(gsm->at_last_sent, get_at(gsm->switchtype,AT_CMIC))) {
						gsm_switch_state(gsm, ALLOGSM_STATE_CLIP_ENABLED, get_at(gsm->switchtype,AT_CLIP_ENABLED));
					}
#ifdef WAVECOM_JUNK
				} else if (gsm_compare(buf, "+CREG: ") || gsm_compare(buf, "+WIND: ") || expectlist_compare(buf)) {
					gsm_send_at(gsm, get_at(gsm->switchtype, AT_CMIC));
#endif
				} else {
					//Freedom del 2012-06-05 15:50
					//gsm_error(gsm, DEBUGFMT "!%s!,last at tx:[%s]\n", DEBUGARGS, buf,gsm->at_last_sent);
				}
				break;

			case ALLOGSM_STATE_CLIP_ENABLED:
#ifdef TESTING
				if(1){
#else
				if (gsm_compare(buf, get_at(gsm->switchtype,AT_OK))) {
#endif
					if (gsm_compare(gsm->at_last_sent, get_at(gsm->switchtype,AT_CLIP_ENABLED))) {
						gsm_switch_state(gsm, ALLOGSM_STATE_RSSI_ENABLED, get_at(gsm->switchtype,AT_RSSI_ENABLED));
					}
#ifdef WAVECOM_JUNK
				} else if (gsm_compare(buf, "+CREG: ") || gsm_compare(buf, "+WIND: ") || expectlist_compare(buf)) {
					gsm_send_at(gsm, get_at(gsm->switchtype, AT_CLIP_ENABLED));
#endif
				} else {
					//Freedom del 2012-06-05 15:50
					//gsm_error(gsm, DEBUGFMT "!%s!,last at tx:[%s]\n", DEBUGARGS, buf,gsm->at_last_sent);
				}
				break;
			case ALLOGSM_STATE_RSSI_ENABLED:
#ifdef TESTING
				if(1){
#else
				if (gsm_compare(buf, get_at(gsm->switchtype,AT_OK))) {
#endif
					if (gsm_compare(gsm->at_last_sent, get_at(gsm->switchtype,AT_RSSI_ENABLED))) {
                                                if (gsm->wind_state & WIND_4_READY_ALL_AT_COMMANDS)
							gsm_switch_state(gsm, ALLOGSM_STATE_SET_NET_URC, get_at(gsm->switchtype,AT_SET_NET_URC));
					}
                                } else if (gsm_compare(buf, "+WIND: 4")) {
					gsm_switch_state(gsm, ALLOGSM_STATE_SET_NET_URC, get_at(gsm->switchtype,AT_SET_NET_URC));

#ifdef WAVECOM_JUNK
				} else if (gsm_compare(buf, "+CREG: ") || gsm_compare(buf, "+WIND: ") || expectlist_compare(buf)) {
						gsm_send_at(gsm, get_at(gsm->switchtype,AT_RSSI_ENABLED));
#endif
				} else {
					//Freedom del 2012-06-05 15:50
					//gsm_error(gsm, DEBUGFMT "!%s!,last at tx:[%s]\n", DEBUGARGS, buf,gsm->at_last_sent);
				}
				break;	
			case ALLOGSM_STATE_SET_NET_URC:
#ifdef TESTING
				if(1){
#else
				if (gsm_compare(buf,  get_at(gsm->switchtype,AT_OK))) {
#endif
					if (gsm_compare(gsm->at_last_sent, get_at(gsm->switchtype,AT_SET_NET_URC))) {
						gsm_switch_state(gsm, ALLOGSM_STATE_NET_REQ, get_at(gsm->switchtype,AT_ASK_NET));
					}
#ifdef WAVECOM_JUNK
				} else if (gsm_compare(buf, "+CREG: ") || gsm_compare(buf, "+WIND: ") || expectlist_compare(buf)) {
						gsm_send_at(gsm, get_at(gsm->switchtype,AT_SET_NET_URC));
#endif
				} else {
					//Freedom del 2012-06-05 15:50
					//gsm_error(gsm, "!%s!\n", buf);
				} 
				break;			
			case ALLOGSM_STATE_NET_REQ:
				if (gsm_compare(gsm->at_last_sent, get_at(gsm->switchtype,AT_ASK_NET))) {
					trim_CRLF(buf);
					if (gsm_compare(buf, get_at(gsm->switchtype,AT_OK))) {						
						if ((GSM_NET_HOME == gsm->network) || (GSM_NET_ROAMING == gsm->network)) {
							gsm_switch_state(gsm, ALLOGSM_STATE_NET_OK, get_at(gsm->switchtype,AT_NET_OK));
						} else {
							gsm->ev.gen.e = ALLOGSM_EVENT_DCHAN_DOWN;
							//sleep (5);
							//gsm_switch_state(gsm, ALLOGSM_STATE_NET_REQ, get_at(gsm->switchtype,AT_ASK_NET)); /* Req Net Status */
							return &gsm->ev;
						}	/* FIXME ---- Do something better here... use a timer... */
					} else if (!strcmp(buf, get_at(gsm->switchtype,AT_CREG10))) {
						gsm_message(gsm,"ALLO GSM: Span: %d UNREGISTERED\n ", gsm->span);
						gsm->network = GSM_NET_UNREGISTERED;
					} else if (!strcmp(buf, get_at(gsm->switchtype,AT_CREG11))) {
						gsm_message(gsm,"ALLO GSM: Span: %d  REGISTERED TO HOME NETWORK\n", gsm->span);
						gsm->network = GSM_NET_HOME;
					} else if (!strcmp(buf, get_at(gsm->switchtype,AT_CREG12))) {
						gsm_message(gsm,"ALLO GSM: Span: %d  SEARCHING FOR NETWORK\n", gsm->span);
						gsm->network = GSM_NET_SEARCHING;
					} else if (!strcmp(buf, get_at(gsm->switchtype,AT_CREG13))) {
						gsm_message(gsm,"ALLO GSM: Span: %d  NETWORK DENIED\n", gsm->span);
						gsm->network = GSM_NET_DENIED;
					} else if (!strcmp(buf, get_at(gsm->switchtype,AT_CREG14))) {
						gsm_message(gsm,"ALLO GSM: Span: %d  UNKNOWN NETWORK\n", gsm->span);
						gsm->network = GSM_NET_UNKNOWN;
					} else if (!strcmp(buf, get_at(gsm->switchtype,AT_CREG15))) {
						gsm_message(gsm,"ALLO GSM: Span: %d  REGISTERED TO ROAMING NETWORK\n", gsm->span);
						gsm->network = GSM_NET_ROAMING;
#ifdef WAVECOM_JUNK
					} else if (gsm_compare(buf, "+CREG: ")) {
						gsm_send_at(gsm, get_at(gsm->switchtype,AT_ASK_NET));
					} else if (gsm_compare(buf, "+WIND: ") || expectlist_compare(buf)) {
						gsm_send_at(gsm, get_at(gsm->switchtype,AT_ASK_NET));
#endif
					} else {
						gsm->network		= GSM_NET_UNREGISTERED;
						gsm->coverage		= -1;
						gsm->net_name[0]	= 0x0;
					}
#ifdef WAVECOM_JUNK
				} else if (gsm_compare(buf, "+CREG: ") || gsm_compare(buf, "+WIND: ") || expectlist_compare(buf)) {
						gsm_send_at(gsm, get_at(gsm->switchtype,AT_ASK_NET));
#endif
				} else {
					if(gsm->ev.gen.e == ALLOGSM_EVENT_DCHAN_DOWN) {
						if(gsm->send_at) {
							gsm_message(gsm, "%s\n", data);
							gsm->send_at = 0;
						}
					} else {
						//Freedom del 2012-06-05 15:50
						//gsm_error(gsm, DEBUGFMT "!%s!,last at tx:[%s]\n", DEBUGARGS, buf,gsm->at_last_sent);
					}
				}
				break;
			case ALLOGSM_STATE_NET_OK:
				if (gsm_compare(buf, get_at(gsm->switchtype,AT_OK))) {
					if (gsm_compare(gsm->at_last_sent, get_at(gsm->switchtype,AT_NET_OK))) { /* set only <format> (for read command +COPS?) C not shown in Read command response */
						gsm_switch_state(gsm, ALLOGSM_STATE_GET_SMSC_REQ, get_at(gsm->switchtype,AT_GET_SMSC));
					}
#ifdef WAVECOM_JUNK
				} else if (gsm_compare(buf, "+CREG: ") || gsm_compare(buf, "+WIND: ") || expectlist_compare(buf)) {
						gsm_send_at(gsm, get_at(gsm->switchtype,AT_NET_OK));
#endif
				}
				break;

			case ALLOGSM_STATE_GET_SMSC_REQ:
				/* Request smsc */
				if (gsm_compare(gsm->at_last_sent, get_at(gsm->switchtype,AT_GET_SMSC))) {
#ifdef TESTING
				if(1){
#else
				if (gsm_compare(buf,get_at(gsm->switchtype,AT_OK))) {
#endif
					gsm_switch_state(gsm, ALLOGSM_STATE_SMS_SET_CHARSET, get_at(gsm->switchtype,AT_SMS_SET_CHARSET));
					} else if (!gsm_compare(buf, get_at(gsm->switchtype,AT_CME_ERROR))) {
						gsm_get_smsc(gsm, buf);
					} else {
						//Freedom del 2012-06-05 15:50
						//gsm_error(gsm, DEBUGFMT "!%s!,last at tx:[%s]\n", DEBUGARGS, buf,gsm->at_last_sent);
					}
#ifdef WAVECOM_JUNK
				} else if (gsm_compare(buf, "+CREG: ") || gsm_compare(buf, "+WIND: ") || expectlist_compare(buf)) {
						gsm_send_at(gsm, get_at(gsm->switchtype,AT_GET_SMSC));
#endif
				} else {
					//Freedom del 2012-06-05 15:50
					//gsm_error(gsm, DEBUGFMT "!%s!,last at tx:[%s]\n", DEBUGARGS, buf,gsm->at_last_sent);
				}
				break;

			case ALLOGSM_STATE_SMS_SET_CHARSET:
#ifdef TESTING
				if(1){
#else
				if (gsm_compare(buf, get_at(gsm->switchtype,AT_OK))) {
#endif
					if(get_at(gsm->switchtype,AT_MODE)){
						gsm_switch_state(gsm, ALLOGSM_AT_MODE, get_at(gsm->switchtype,AT_MODE));
					} else if (gsm_compare(gsm->at_last_sent, get_at(gsm->switchtype,AT_SMS_SET_CHARSET))) {
						gsm_switch_state(gsm, ALLOGSM_STATE_SMS_SET_INDICATION, get_at(gsm->switchtype,AT_GSM));
					}
#ifdef WAVECOM_JUNK
				} else if (gsm_compare(buf, "+CREG: ") || gsm_compare(buf, "+WIND: ") || expectlist_compare(buf)) {
						gsm_send_at(gsm, get_at(gsm->switchtype,AT_SMS_SET_CHARSET));
#endif
				} else {
					//Freedom del 2012-06-05 15:50
					//gsm_error(gsm, DEBUGFMT "!%s!,last at tx:[%s]\n", DEBUGARGS, buf,gsm->at_last_sent);
				}
				break;

			case ALLOGSM_AT_MODE:
#ifdef TESTING
				if(1){
#else
				if (gsm_compare(buf, get_at(gsm->switchtype,AT_OK))) {
#endif
					if (gsm_compare(gsm->at_last_sent, get_at(gsm->switchtype,AT_MODE))) { /*add by makes 2012-4-10 15:54*/
						gsm_switch_state(gsm, ALLOGSM_STATE_SMS_SET_INDICATION, get_at(gsm->switchtype,AT_GSM));
					}
#ifdef WAVECOM_JUNK
				} else if (gsm_compare(buf, "+CREG: ") || gsm_compare(buf, "+WIND: ") || expectlist_compare(buf)) {
						gsm_send_at(gsm, get_at(gsm->switchtype,AT_MODE));
#endif
				}
				break;

			case ALLOGSM_STATE_SMS_SET_INDICATION:
#ifdef TESTING
				if(1){
#else
				if (gsm_compare(buf,get_at(gsm->switchtype,AT_OK))) {
#endif
					if (gsm_compare(gsm->at_last_sent, get_at(gsm->switchtype,AT_GSM))) {
						gsm->sms_mod_flag = SMS_TEXT;
						sleep(1);
						gsm_switch_state(gsm, ALLOGSM_STATE_NET_NAME_REQ, get_at(gsm->switchtype,AT_ASK_NET_NAME));
					}
#ifdef WAVECOM_JUNK
				} else if (gsm_compare(buf, "+CREG: ") || gsm_compare(buf, "+WIND: ") || expectlist_compare(buf)) {
						gsm_send_at(gsm, get_at(gsm->switchtype,AT_GSM));
#endif
				} else {
					//Freedom del 2012-06-05 15:50
					//gsm_error(gsm, DEBUGFMT "!%s!,last at tx:[%s]\n", DEBUGARGS, buf,gsm->at_last_sent);
				}
				break;
			case ALLOGSM_STATE_NET_NAME_REQ:
				if (gsm_compare(buf, get_at(gsm->switchtype,AT_CHECK_NET))) {
					if (gsm_compare(gsm->at_last_sent, get_at(gsm->switchtype,AT_ASK_NET_NAME))) {
						gsm_get_operator(gsm, buf);
						UPDATE_OURCALLSTATE(gsm, call, AT_CALL_STATE_NULL);
						call->peercallstate = AT_CALL_STATE_NULL;
#ifdef VIRTUAL_TTY
						if(gsm->already_set_mux_mode) {
							gsm_switch_state(gsm, ALLOGSM_STATE_NET_NAME_REQ, get_at(gsm->switchtype,AT_NET_NAME));
						} else {
							gsm_switch_state(gsm, ALLOGSM_INIT_MUX, get_at(gsm->switchtype,AT_CHECK));
						}
#else
						gsm_switch_state(gsm, ALLOGSM_STATE_NET_NAME_REQ, get_at(gsm->switchtype,AT_NET_NAME));
#endif
						if (gsm->retranstimer) {
							gsm_schedule_del(gsm, gsm->retranstimer);
							gsm->retranstimer = 0;
						}
						if (gsm->resetting) {
							gsm->resetting = 0;
							gsm->ev.gen.e = ALLOGSM_EVENT_RESTART_ACK;
						} else {
							gsm->ev.gen.e = ALLOGSM_EVENT_DCHAN_UP;
						}
						return &gsm->ev;
					}
#ifdef WAVECOM_JUNK
				} else if (gsm_compare(buf, "+CREG: ") || gsm_compare(buf, "+WIND: ") || expectlist_compare(buf)) {
						gsm_send_at(gsm, get_at(gsm->switchtype,AT_ASK_NET_NAME));
#endif
				} else {
					//Freedom del 2012-06-05 15:50
					//gsm_error(gsm, DEBUGFMT "!%s!,last at tx:[%s]\n", DEBUGARGS, buf,gsm->at_last_sent);
				}
				break;
#ifdef VIRTUAL_TTY
			case ALLOGSM_INIT_MUX:
				gsm->ev.gen.e = ALLOGSM_EVENT_INIT_MUX;
				return &gsm->ev;
				break;
#endif //VIRTUAL_TTY

			case ALLOGSM_STATE_READY:
				/* Request operators */
				if (gsm_compare(buf, get_at(gsm->switchtype,AT_CHECK_NET))) { /* AT+COPS? */
					//printf("cops buf: >>%s<<\n", buf);
					strncpy(gsm->net_name, buf, sizeof(gsm->net_name));
			//	} else if (gsm_compare(buf, get_at(gsm->switchtype,AT_OK))) {
					//Freedom notice must to rewrite this code 2011-10-12 15:13
			//		gsm_message(gsm, "%s\n", data);
				} else if (gsm_compare(buf, get_at(gsm->switchtype,AT_RING))) { /* RING  */
					/*This condition checks RINGS should not exeed a count of 2.
					  Coz after a RING, CLIP event comes.
					  If CLIP doesnt comes, assume no caller id gonna come and 
					  we gonna make a fake CLIP EVENT and goto next condition

					If anytime we are getting a call with UNKNOWN number 
					1st check CLIP is coming or not, If coming debug here otherwise everything is cool 
					*/
                                        if (call->newcall){
                                                call->ring_count=call->ring_count+1;
                                                if (call->ring_count>1){
							/* lets build CLIP RESPONSE */
							memset(buf, 0x0, sizeof(buf));
							sprintf(buf, "+CLIP: \"UNKNOWN\",129,1,,\"UNKNOWN\"");
                                                        goto call_without_callerid;
						}
                                        }
				} else if (gsm_compare(buf, get_at(gsm->switchtype,AT_INCOMING_CALL))) { /* Incoming call */
call_without_callerid:
					/*If getting a CLIP event for first time, then only go inside this condition otherwise break*/
					if (!call->newcall) {
						break;
					}
					call->newcall = 0;

					char caller_id[64];
					/* An unsolicited result code is returned after every RING at a mobile terminating call  */
					/* +CLIP: <number>, <type>,,,<alphaId>,<CLI validity> */
					get_cid(gsm->switchtype,buf,caller_id,sizeof(caller_id));

					/* Set caller number and caller name (if provided) */
					if (!strlen(caller_id)) {
						printf("kkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkk\n");
						strncpy(gsm->ev.ring.callingnum, "", sizeof(gsm->ev.ring.callingnum));
					} else {
						strncpy(gsm->ev.ring.callingnum, caller_id, sizeof(gsm->ev.ring.callingnum));
						strncpy(gsm->ev.ring.callingname, caller_id, sizeof(gsm->ev.ring.callingname));
					}

					/* Return ring event */
					UPDATE_OURCALLSTATE(gsm, call, AT_CALL_STATE_CALL_PRESENT);
					call->peercallstate = AT_CALL_STATE_CALL_INITIATED;
					call->alive = 1;
					gsm->state = ALLOGSM_STATE_RING;
					gsm->ev.e					= ALLOGSM_EVENT_RING;
					gsm->ev.ring.channel		= call->channelno; /* -1 : default */
					gsm->ev.ring.cref			= call->cr;
					gsm->ev.ring.call			= call;
					gsm->ev.ring.layer1			= ALLOGSM_LAYER_1_ALAW; /* a law */
					gsm->ev.ring.complete		= call->complete; 
					gsm->ev.ring.progress		= call->progress;
					gsm->ev.ring.progressmask	= call->progressmask;
#if 1
					gsm->ev.ring.callednum[0]	= '\0';				/* called number should not be existed */ 
#else
                                        char tmp[4] ; 
                                        sprintf (tmp, "%d", gsm->span) ;
					strncpy(gsm->ev.ring.callednum, tmp, sizeof(gsm->ev.ring.callednum));
#endif
				//	gsm_switch_state(gsm, ALLOGSM_STATE_RING, get_at(gsm->switchtype,AT_RING));//sujay
					return &gsm->ev;
#ifdef WAVECOM
/*  Should not call AT+CSQ(AT_NET_NAME) here..
                                } else if (gsm_compare(buf, "+CREG: ") || gsm_compare(buf, "+WIND: 4") || gsm_compare(buf, "+WIND: 7")) {
                                                gsm_send_at(gsm, get_at(gsm->switchtype,AT_NET_NAME));
*/
#endif
				} else {
					//gsm_message(gsm, "send_at:%d %s\n", gsm->send_at,data);
					if(gsm->send_at) {
						gsm_message(gsm, "%s\n", data);
						gsm->send_at = 0;
					}
				}
				break;
			case ALLOGSM_STATE_RING:
				if (gsm_compare(buf, get_at(gsm->switchtype,AT_RING))) {
					call->alive = 1;
					gsm->state = ALLOGSM_STATE_RINGING;
					gsm->ev.e			= ALLOGSM_EVENT_RINGING;
					gsm->ev.ring.channel		= call->channelno;
					gsm->ev.ring.cref		= call->cr;
					gsm->ev.ring.progress		= call->progress;
					gsm->ev.ring.progressmask	= call->progressmask;
					return &gsm->ev;
				}
				break;
			//Freedom Add 2011-12-08 15:32 Check reject call
			////////////////////////////////////////////////////
			case ALLOGSM_STATE_RINGING:
#ifdef WAVECOM
				if( gsm_compare(buf, get_at(gsm->switchtype,AT_NO_CARRIER)) ||
					gsm_compare(buf, get_at(gsm->switchtype,AT_NO_ANSWER)) || gsm_compare(buf, "+WIND: 6,1") ) {
#else				
				if( gsm_compare(buf, get_at(gsm->switchtype,AT_NO_CARRIER)) ||
					gsm_compare(buf, get_at(gsm->switchtype,AT_NO_ANSWER)) ) {
#endif					
					gsm->state = ALLOGSM_STATE_READY;
					UPDATE_OURCALLSTATE(gsm, call, AT_CALL_STATE_NULL);
					call->peercallstate = AT_CALL_STATE_NULL;
					gsm->ev.e = ALLOGSM_EVENT_HANGUP;
					gsm->ev.hangup.channel = call->channelno;
					gsm->ev.hangup.cause = ALLOGSM_CAUSE_NORMAL_CLEARING;
					gsm->ev.hangup.cref = call->cr;
					gsm->ev.hangup.call = call;
					call->alive = 0;
					call->sendhangupack = 0;
					allogsm_destroycall(gsm, call);
					return &gsm->ev;
				}
				break;
			////////////////////////////////////////////////////
			case ALLOGSM_STATE_PRE_ANSWER:
				/* Answer the remote calling */
				if (gsm_compare(buf, get_at(gsm->switchtype,AT_OK))) {
					if (gsm_compare(gsm->at_last_sent, get_at(gsm->switchtype,AT_ANSWER))) {
						UPDATE_OURCALLSTATE(gsm, call, AT_CALL_STATE_ACTIVE);
						call->peercallstate = AT_CALL_STATE_ACTIVE;
						call->alive = 1;
						gsm->state = ALLOGSM_STATE_CALL_ACTIVE;
						gsm->ev.e = ALLOGSM_EVENT_ANSWER;
						gsm->ev.answer.progress	= 0;
						gsm->ev.answer.channel	= call->channelno;
						gsm->ev.answer.cref		= call->cr;
						gsm->ev.answer.call		= call;
						return &gsm->ev;
					}
				}
				break;
			case ALLOGSM_STATE_CALL_ACTIVE:
				/* Remote end of active all. Waiting ...*/
#ifdef WAVECOM
				if (gsm_compare(buf, get_at(gsm->switchtype,AT_NO_CARRIER)) || 
					gsm_compare(buf, get_at(gsm->switchtype,AT_NO_ANSWER)) ||  gsm_compare(buf,"+WIND: 6,1") 
					||  gsm_compare(buf,"BUSY") || gsm_compare(buf,"+CREG: ")) {
#else
				if (gsm_compare(buf, get_at(gsm->switchtype,AT_NO_CARRIER)) || 
					gsm_compare(buf, get_at(gsm->switchtype,AT_NO_ANSWER))) {
#endif
					int cause = 16;

					if (gsm_compare(buf, get_at(gsm->switchtype,AT_NO_CARRIER)))
						cause	= ALLOGSM_CAUSE_NORMAL_CLEARING;
					else if (gsm_compare(buf, get_at(gsm->switchtype,AT_NO_ANSWER)))
						cause	= ALLOGSM_CAUSE_NO_ANSWER;
					else if (gsm_compare(buf, "BUSY"))
						cause	= ALLOGSM_CAUSE_USER_BUSY;
					else if (gsm_compare(buf, "CREG: "))
						cause	= ALLOGSM_CAUSE_NETWORK_OUT_OF_ORDER;

					gsm_switch_state(gsm, ALLOGSM_STATE_READY, get_at(gsm->switchtype,AT_NET_NAME));
					UPDATE_OURCALLSTATE(gsm, call, AT_CALL_STATE_NULL);
					call->peercallstate = AT_CALL_STATE_NULL;
					call->alive = 0;
					call->sendhangupack = 0;
					gsm->ev.e				= ALLOGSM_EVENT_HANGUP;
					gsm->ev.hangup.channel	= 1;
					gsm->ev.hangup.cause		= cause;
					gsm->ev.hangup.cref		= call->cr;
					gsm->ev.hangup.call		= call;
					allogsm_hangup(gsm, call, cause);
					return &gsm->ev;
				}
#if 1
				if (gsm_compare(buf, "+WDDI")){

					char *dtmf_digit;
					int duration;
					char *pbuf;
					pbuf= buf;
					strsep(&pbuf, "\"");
					dtmf_digit=(char *)strsep(&pbuf, "\"");
					strsep(&pbuf, ",");
					duration=atoi(pbuf);
					gsm_message(gsm,"DTMF%s Duration:%d\n", dtmf_digit, duration);

                                        gsm->ev.e              = ALLOGSM_EVENT_KEYPAD_DIGIT;
                                        gsm->ev.digit.channel  = 1;
                                        gsm->ev.digit.call     = call;
                                        sprintf(gsm->ev.digit.digits, "%s",dtmf_digit);
#define IGNORE_LESS_DURATION_DTMF 1
#ifdef IGNORE_LESS_DURATION_DTMF
					if (duration<80)
						gsm_message(gsm,"Ignoring DTMF coz of duration %d\n", duration);
					else
                                        	return &gsm->ev;
#else
                                        return &gsm->ev;
#endif
				}
#endif
				break;
			case ALLOGSM_STATE_HANGUP_REQ:
				/* Hangup the active call */
				if (gsm_compare(buf, get_at(gsm->switchtype,AT_OK))) {
junk_received_for_ATH:
					if (gsm_compare(gsm->at_last_sent, get_at(gsm->switchtype,AT_HANGUP))) {
						UPDATE_OURCALLSTATE(gsm, call, AT_CALL_STATE_NULL);
						call->peercallstate		= AT_CALL_STATE_NULL;
						call->alive = 0;
						call->sendhangupack = 0;
						gsm->state = ALLOGSM_STATE_READY;
						gsm->ev.e				= ALLOGSM_EVENT_HANGUP;
						gsm->ev.hangup.channel	= 1;
						gsm->ev.hangup.cause	= ALLOGSM_CAUSE_NORMAL_CLEARING; // trabajar el c->cause con el ^CEND
						gsm->ev.hangup.cref		= call->cr;
						gsm->ev.hangup.call		= call;
#ifdef WAVECOM
						//gsm_send_at(gsm, "AT+WIND=0");
#endif
						return &gsm->ev;
					}
				} else if (gsm_compare(buf, get_at(gsm->switchtype,AT_NO_CARRIER)) || 
					gsm_compare(buf, get_at(gsm->switchtype,AT_NO_ANSWER)) ||  gsm_compare(buf,"+WIND: 6,1") 
					||  gsm_compare(buf,"BUSY") || gsm_compare(buf,"+CREG: ")) {
					return NULL;

#ifdef WAVECOM
				} else if (gsm_compare(buf, "+CREG: ") || gsm_compare(buf, "+WIND: ")) {
						gsm_send_at(gsm, get_at(gsm->switchtype,AT_HANGUP));
				} else if (expectlist_compare(buf)) {
					goto junk_received_for_ATH;
#endif
				}
				break;
			case ALLOGSM_STATE_CALL_INIT:
				/* After dial */
#ifdef WAVECOM
                                if (gsm_compare(buf, "+WIND: 5,")) {
//                                      gsm->state = ALLOGSM_STATE_CALL_MADE;
                                        call->channelno = 1;
//                                        gsm_switch_state(gsm, ALLOGSM_STATE_CALL_PROCEEDING, get_at(gsm->switchtype,AT_CALL_PROCEEDING));
                                        gsm_switch_state(gsm, ALLOGSM_STATE_CALL_PROCEEDING, NULL);
                                        gsm->ev.e                       = ALLOGSM_EVENT_PROCEEDING;
                                        gsm->ev.proceeding.progress     = 8;
                                        gsm->ev.proceeding.channel      = call->channelno;
                                        gsm->ev.proceeding.cause        = 0;
                                        gsm->ev.proceeding.cref         = call->cr;
                                        gsm->ev.proceeding.call         = call;
                                        return &gsm->ev;
				} else if (gsm_compare(buf, "+CREG: 1")){
					/*Added for reset bug*/
                                        if (gsm_compare(gsm->at_last_sent, "ATD")) {
						gsm_send_at(gsm, gsm->at_last_sent);
					}
                                }else{
                                        if (gsm_compare(gsm->at_last_sent, "ATD")) {
                                               // gsm_switch_state(gsm, ALLOGSM_STATE_HANGUP_REQ, get_at(gsm->switchtype,AT_HANGUP));
                                        }
                                }
#else
//				if (gsm_compare(buf, get_at(gsm->switchtype,AT_OK))) {
						gsm_send_at(gsm, get_at(gsm->switchtype,AT_CALL_INIT));
						gsm->state = ALLOGSM_STATE_CALL_MADE;
//				}
#endif
				break;
			case ALLOGSM_STATE_CALL_MADE:
#ifdef WAVECOM
#else
			//	if (gsm_compare(buf, get_at(gsm->switchtype,AT_OK))) {
					if (gsm_compare(gsm->at_last_sent, get_at(gsm->switchtype,AT_CALL_INIT))) {
						call->channelno = 1;
						//gsm_switch_state(gsm, ALLOGSM_STATE_CALL_PROCEEDING, get_at(gsm->switchtype,AT_CALL_PROCEEDING));
						gsm_switch_state(gsm, ALLOGSM_STATE_CALL_PROCEEDING, NULL);
						gsm->ev.e 					= ALLOGSM_EVENT_PROCEEDING;
						gsm->ev.proceeding.progress	= 8;
						gsm->ev.proceeding.channel	= call->channelno;
						gsm->ev.proceeding.cause	= 0;
						gsm->ev.proceeding.cref		= call->cr;
						gsm->ev.proceeding.call		= call;
						return &gsm->ev;
					}
				//}
#endif
				break;
			case ALLOGSM_STATE_CALL_PROCEEDING:
#ifdef WAVECOM
                                if (gsm_compare(buf, "+WIND: 2")) {
                                        //Freedom Add 2012-02-07 15:24
                                        //////////////////////////////////////////////////////////////////////////
#if SIMCOM900D_NO_ANSWER_BUG
                                        first = 1;
#endif //SIMCOM900D_NO_ANSWER_BUG
                                        //////////////////////////////////////////////////////////////////////////
                                        gsm->state = ALLOGSM_STATE_CALL_PROGRESS;
                                        gsm->ev.proceeding.e            = ALLOGSM_EVENT_PROGRESS;
                                        gsm->ev.proceeding.progress     = 8;
                                        gsm->ev.proceeding.channel      = call->channelno;
                                        gsm->ev.proceeding.cause        = 0;
                                        gsm->ev.proceeding.cref         = call->cr;
                                        gsm->ev.proceeding.call         = call;
                                        return &gsm->ev;
                                } else if (gsm_compare(buf, "OK")) {
					goto auto_answer_without_progress;
#if 0
				} else if (gsm_compare(buf, "+CREG: 1")){
					/*Added for reset bug*/
                                        if (gsm_compare(gsm->at_last_sent, "ATD")) {
						gsm_send_at(gsm, gsm->at_last_sent);
					}
#endif
                                } else {
                                        if (gsm_compare(gsm->at_last_sent, "ATD")) {
//						gsm_switch_state(gsm, ALLOGSM_STATE_HANGUP_REQ, get_at(gsm->switchtype,AT_HANGUP));
                                        }
                                }
#else
				//if (gsm_compare(buf, get_at(gsm->switchtype,AT_OK))) {
				//	if (gsm_compare(gsm->at_last_sent, get_at(gsm->switchtype,AT_CALL_PROCEEDING))) {
						
						//Freedom Add 2012-02-07 15:24
						//////////////////////////////////////////////////////////////////////////
#if SIMCOM900D_NO_ANSWER_BUG
						first = 1;
#endif //SIMCOM900D_NO_ANSWER_BUG
						//////////////////////////////////////////////////////////////////////////
						gsm->state = ALLOGSM_STATE_CALL_PROGRESS;
						gsm->ev.proceeding.e		= ALLOGSM_EVENT_PROGRESS;
						gsm->ev.proceeding.progress	= 8;
						gsm->ev.proceeding.channel	= call->channelno;
						gsm->ev.proceeding.cause	= 0;
						gsm->ev.proceeding.cref		= call->cr;
						gsm->ev.proceeding.call		= call;
						return &gsm->ev;
				//	}
				//}
#endif
				break;
			case ALLOGSM_STATE_CALL_PROGRESS:
				//Freedom Add 2012-02-07 15:24
				//////////////////////////////////////////////////////////////////////////
#if SIMCOM900D_NO_ANSWER_BUG
				if(first) {
					first = 0;
					gettimeofday(&start_time,NULL);
				} else {
					gettimeofday(&end_time,NULL);
					if((end_time.tv_sec-start_time.tv_sec) >= 30 ) {
						first = 1;
						gsm_message(gsm,"Dial Timeout\n");
						gsm->state = ALLOGSM_STATE_READY;
						UPDATE_OURCALLSTATE(gsm, call, AT_CALL_STATE_NULL);
						call->peercallstate = AT_CALL_STATE_NULL;
						gsm->ev.e = ALLOGSM_EVENT_HANGUP;
						gsm->ev.hangup.channel = call->channelno;
						gsm->ev.hangup.cause = ALLOGSM_CAUSE_NO_ANSWER;
						gsm->ev.hangup.cref = call->cr;
						gsm->ev.hangup.call = call;
						call->alive = 0;
						call->sendhangupack = 0;
						module_hangup(gsm, call);
						allogsm_destroycall(gsm, call);
						return &gsm->ev;
					}
				}
#endif //SIMCOM900D_NO_ANSWER_BUG
				//////////////////////////////////////////////////////////////////////////
#ifdef WAVECOM
auto_answer_without_progress:
                                        if (gsm_compare(buf, "OK")) {
                                                call->alive = 1;
                                                gsm->state = ALLOGSM_STATE_CALL_ACTIVE;
//                                              gsm->state = ALLOGSM_STATE_PRE_ANSWER; // If AT_MO_CONNECTED is not proper, comment if case and use this state
                                                gsm->ev.gen.e = ALLOGSM_EVENT_ANSWER;
                                                gsm->ev.answer.progress = 0;
                                                gsm->ev.answer.channel = call->channelno;
                                                gsm->ev.answer.cref = call->cr;
                                                gsm->ev.answer.call = call;
                                                return &gsm->ev;
                                        } else if( gsm_compare(buf, get_at(gsm->switchtype,AT_NO_CARRIER)) ||
                                        gsm_compare(buf, get_at(gsm->switchtype,AT_NO_ANSWER)) || gsm_compare(buf, "+WIND: 6,1") ) {
						int cause = 19;

						if (gsm_compare(buf, get_at(gsm->switchtype,AT_NO_CARRIER)))
							cause	= ALLOGSM_CAUSE_USER_BUSY;
						else if (gsm_compare(buf, get_at(gsm->switchtype,AT_NO_ANSWER)))
							cause	= ALLOGSM_CAUSE_NO_ANSWER;
						else if (gsm_compare(buf, "BUSY"))
							cause	= ALLOGSM_CAUSE_USER_BUSY;

                                                gsm->state = ALLOGSM_STATE_READY; 
                                                UPDATE_OURCALLSTATE(gsm, call, AT_CALL_STATE_NULL);
                                                call->peercallstate = AT_CALL_STATE_NULL;
                                                gsm->ev.e = ALLOGSM_EVENT_HANGUP;
                                                gsm->ev.hangup.channel = call->channelno;
                                                gsm->ev.hangup.cause = cause;
                                                gsm->ev.hangup.cref = call->cr;
                                                gsm->ev.hangup.call = call;
                                                call->alive = 0;
                                                call->sendhangupack = 0;
                                                allogsm_destroycall(gsm, call);
                                                return &gsm->ev;
                                        }
#else
//	     				if (gsm_compare(buf, get_at(gsm->switchtype,AT_MO_CONNECTED))) {
						call->alive = 1;
				//		gsm->state = ALLOGSM_STATE_CALL_ACTIVE;
						gsm->state = ALLOGSM_STATE_PRE_ANSWER; // If AT_MO_CONNECTED is not proper, comment if case and use this state
						gsm->ev.gen.e = ALLOGSM_EVENT_ANSWER;
						gsm->ev.answer.progress = 0;
						gsm->ev.answer.channel = call->channelno;
						gsm->ev.answer.cref = call->cr;
						gsm->ev.answer.call = call;
						return &gsm->ev;
//				} 
#endif
				break;
#ifdef CONFIG_CHECK_PHONE
			case ALLOGSM_STATE_PHONE_CHECK: /*add by makes 2012-04-10 11:03 */
			{
				if(time(NULL)<gsm->check_timeout){
					if (gsm_compare(buf, get_at(gsm->switchtype,AT_RING))){
						if(gsm->auto_hangup_flag)
							gsm_switch_state(gsm, ALLOGSM_STATE_HANGUP_REQ, get_at(gsm->switchtype,AT_HANGUP));
						gsm->ev.gen.e = ALLOGSM_EVENT_CHECK_PHONE;
						gsm->ev.notify.info = PHONE_RING;
						gsm->phone_stat=PHONE_RING;
						return &gsm->ev;
					}
					else if(gsm_compare(buf, get_at(gsm->switchtype,AT_BUSY))){
						gsm_switch_state(gsm, ALLOGSM_STATE_HANGUP_REQ, get_at(gsm->switchtype,AT_HANGUP));
						gsm->ev.gen.e = ALLOGSM_EVENT_CHECK_PHONE;
						gsm->ev.notify.info = PHONE_BUSY;
						gsm->phone_stat=PHONE_BUSY;
						return &gsm->ev;
					}
					else if(gsm_compare(buf, get_at(gsm->switchtype,AT_MO_CONNECTED))){
						gsm_switch_state(gsm, ALLOGSM_STATE_HANGUP_REQ, get_at(gsm->switchtype,AT_HANGUP));
						gsm->ev.gen.e = ALLOGSM_EVENT_CHECK_PHONE;
						gsm->ev.notify.info = PHONE_CONNECT;
						gsm->phone_stat=PHONE_CONNECT;
						return &gsm->ev;
					}
					else if(gsm_compare(buf, get_at(gsm->switchtype,AT_NO_CARRIER))){
						gsm_switch_state(gsm, ALLOGSM_STATE_HANGUP_REQ, get_at(gsm->switchtype,AT_HANGUP));
						gsm->ev.gen.e = ALLOGSM_EVENT_CHECK_PHONE;
						gsm->ev.notify.info = PHONE_NOT_CARRIER;
						gsm->phone_stat=PHONE_NOT_CARRIER;
						return &gsm->ev;
					}
					else if(gsm_compare(buf, get_at(gsm->switchtype,AT_NO_ANSWER))){
						gsm_switch_state(gsm, ALLOGSM_STATE_HANGUP_REQ, get_at(gsm->switchtype,AT_HANGUP));
						gsm->ev.gen.e = ALLOGSM_EVENT_CHECK_PHONE;
						gsm->ev.notify.info = PHONE_NOT_ANSWER;
						gsm->phone_stat=PHONE_NOT_ANSWER;
						return &gsm->ev;
					}
					else if(gsm_compare(buf, get_at(gsm->switchtype,AT_NO_DIALTONE))){
						gsm_switch_state(gsm, ALLOGSM_STATE_HANGUP_REQ, get_at(gsm->switchtype,AT_HANGUP));
						gsm->ev.gen.e = ALLOGSM_EVENT_CHECK_PHONE;
						gsm->ev.notify.info = PHONE_NOT_DIALTONE;
						gsm->phone_stat=PHONE_NOT_DIALTONE;
						return &gsm->ev;
					}
#ifdef WAVECOM
					else if(gsm_compare(buf, "+WIND: 6,1")){
						gsm_switch_state(gsm, ALLOGSM_STATE_HANGUP_REQ, get_at(gsm->switchtype,AT_HANGUP));
						gsm->ev.gen.e = ALLOGSM_EVENT_CHECK_PHONE;
						gsm->ev.notify.info = PHONE_NOT_CARRIER;
						gsm->phone_stat=PHONE_NOT_CARRIER;
						return &gsm->ev;
					}
#endif 
				}
				else{
					if(gsm->auto_hangup_flag)
					{
						gsm_switch_state(gsm, ALLOGSM_STATE_HANGUP_REQ, get_at(gsm->switchtype,AT_HANGUP));
						gsm->ev.gen.e = ALLOGSM_EVENT_CHECK_PHONE;
						gsm->ev.notify.info = PHONE_TIMEOUT;
						gsm->phone_stat=PHONE_TIMEOUT;
						return &gsm->ev;
					}
					else
					{
						if(gsm_compare(buf, get_at(gsm->switchtype,AT_BUSY))){
							gsm_switch_state(gsm, ALLOGSM_STATE_HANGUP_REQ, get_at(gsm->switchtype,AT_HANGUP));
							gsm->ev.gen.e = ALLOGSM_EVENT_CHECK_PHONE;
							gsm->ev.notify.info = PHONE_BUSY;
							gsm->phone_stat=PHONE_BUSY;
							return &gsm->ev;
						}
						else if(gsm_compare(buf, get_at(gsm->switchtype,AT_MO_CONNECTED))){
							gsm_switch_state(gsm, ALLOGSM_STATE_HANGUP_REQ, get_at(gsm->switchtype,AT_HANGUP));
							gsm->ev.gen.e = ALLOGSM_EVENT_CHECK_PHONE;
							gsm->ev.notify.info = PHONE_CONNECT;
							gsm->phone_stat=PHONE_CONNECT;
							return &gsm->ev;
						}
						else if(gsm_compare(buf, get_at(gsm->switchtype,AT_NO_CARRIER))){
							gsm_switch_state(gsm, ALLOGSM_STATE_HANGUP_REQ, get_at(gsm->switchtype,AT_HANGUP));
							gsm->ev.gen.e = ALLOGSM_EVENT_CHECK_PHONE;
							gsm->ev.notify.info = PHONE_NOT_CARRIER;
							gsm->phone_stat=PHONE_NOT_CARRIER;
							return &gsm->ev;
						}
						else if(gsm_compare(buf, get_at(gsm->switchtype,AT_NO_ANSWER))){
							gsm_switch_state(gsm, ALLOGSM_STATE_HANGUP_REQ, get_at(gsm->switchtype,AT_HANGUP));
							gsm->ev.gen.e = ALLOGSM_EVENT_CHECK_PHONE;
							gsm->ev.notify.info = PHONE_NOT_ANSWER;
							gsm->phone_stat=PHONE_NOT_ANSWER;
							return &gsm->ev;
						}
						else if(gsm_compare(buf, get_at(gsm->switchtype,AT_NO_DIALTONE))){
							gsm_switch_state(gsm, ALLOGSM_STATE_HANGUP_REQ, get_at(gsm->switchtype,AT_HANGUP));
							gsm->ev.gen.e = ALLOGSM_EVENT_CHECK_PHONE;
							gsm->ev.notify.info = PHONE_NOT_DIALTONE;
							gsm->phone_stat=PHONE_NOT_DIALTONE;
							return &gsm->ev;
						}
#ifdef WAVECOM
						else if(gsm_compare(buf, "+WIND: 6,1")){
							gsm_switch_state(gsm, ALLOGSM_STATE_HANGUP_REQ, get_at(gsm->switchtype,AT_HANGUP));
							gsm->ev.gen.e = ALLOGSM_EVENT_CHECK_PHONE;
							gsm->ev.notify.info = PHONE_NOT_CARRIER;
							gsm->phone_stat=PHONE_NOT_CARRIER;
							return &gsm->ev;
						}
#endif 
					}
				}
				break;
			}
#endif
			default:
				break;
		}

		i ++;
				
		if(gsm->state >= ALLOGSM_STATE_READY) {
#ifdef WAVECOM
			if(gsm_compare(buf, get_at(gsm->switchtype,AT_NO_CARRIER)) ||
				gsm_compare(buf, get_at(gsm->switchtype,AT_NO_ANSWER)) || 
				gsm_compare(buf, "BUSY") || 
				gsm_compare(buf, "+WIND: 6,1")) {
#else
			if(gsm_compare(buf, get_at(gsm->switchtype,AT_NO_CARRIER)) ||
				gsm_compare(buf, get_at(gsm->switchtype,AT_NO_ANSWER))) {
#endif
				int cause = 19;

				if (gsm_compare(buf, get_at(gsm->switchtype,AT_NO_CARRIER)))
					cause	= ALLOGSM_CAUSE_USER_BUSY;
				else if (gsm_compare(buf, get_at(gsm->switchtype,AT_NO_ANSWER)))
					cause	= ALLOGSM_CAUSE_NO_ANSWER;
				else if (gsm_compare(buf, "BUSY"))
					cause	= ALLOGSM_CAUSE_USER_BUSY;

				gsm->state = ALLOGSM_STATE_READY;
				UPDATE_OURCALLSTATE(gsm, call, AT_CALL_STATE_NULL);
				call->peercallstate = AT_CALL_STATE_NULL;
				gsm->ev.e = ALLOGSM_EVENT_HANGUP;
				gsm->ev.hangup.channel = call->channelno;
				gsm->ev.hangup.cause = cause;
				gsm->ev.hangup.cref = call->cr;
				gsm->ev.hangup.call = call;
				call->alive = 0;
				call->sendhangupack = 0;
				allogsm_destroycall(gsm, call);
				return &gsm->ev;
			} else if (gsm_compare(buf, get_at(gsm->switchtype,AT_NO_DIALTONE))) {
				gsm->state = ALLOGSM_STATE_READY;
				UPDATE_OURCALLSTATE(gsm, call, AT_CALL_STATE_NULL);
				call->peercallstate = AT_CALL_STATE_NULL;
				gsm->ev.e = ALLOGSM_EVENT_HANGUP;
				gsm->ev.hangup.cause = ALLOGSM_CAUSE_NETWORK_OUT_OF_ORDER;
				gsm->ev.hangup.cref = call->cr;
				gsm->ev.hangup.call = call;
				call->alive = 0;
				call->sendhangupack = 0;
				allogsm_destroycall(gsm, call);
				return &gsm->ev;
			} else if (gsm_compare(buf, get_at(gsm->switchtype,AT_BUSY))) {
				gsm->state = ALLOGSM_STATE_READY;
				UPDATE_OURCALLSTATE(gsm, call, AT_CALL_STATE_NULL);
				call->peercallstate = AT_CALL_STATE_NULL;
				gsm->ev.e = ALLOGSM_EVENT_HANGUP;
				gsm->ev.hangup.channel = call->channelno;
				gsm->ev.hangup.cause = ALLOGSM_CAUSE_USER_BUSY;
				gsm->ev.hangup.cref = call->cr;
				gsm->ev.hangup.call = call;
				call->alive = 0;
				call->sendhangupack = 0;
				allogsm_destroycall(gsm, call);
				return &gsm->ev;
			}
		}
	
#if 0	
		res_event = module_check_wind(gsm, call ,buf, i);
		if (res_event) {
			return res_event;
		}
#endif
		
		res_event = module_check_sms(gsm, buf, i);
		if (res_event) {
			return res_event;
		}

		res_event = module_check_network(gsm, call ,buf, i);
		if (res_event) {
			return res_event;
		}

		res_event = module_check_ussd(gsm, buf, i);
		if (res_event) {
			return res_event;
		}
	}

	return res_event;
}

#ifdef CONFIG_CHECK_PHONE
void module_hangup_phone(struct allogsm_modul *gsm)
{
	gsm_switch_state(gsm, ALLOGSM_STATE_HANGUP_REQ, get_at(gsm->switchtype,AT_HANGUP));
}

int module_check_phone_stat(struct allogsm_modul *gsm, char *phone_number,int hangup_flag,unsigned int timeout)
{
	char buf[128];
	int time_out=0;
	memset(buf, 0x0, sizeof(buf));
	gsm->phone_stat=-1;
	gsm->auto_hangup_flag=hangup_flag;
	if(timeout<=0)
		time_out=DEFAULT_CHECK_TIMEOUT;
	else
		time_out=timeout;
	if(gsm->state!=ALLOGSM_STATE_READY){
		return -1;
	}
	else
	{
		gsm->check_timeout=time(NULL)+time_out;
		get_dial_str(gsm->switchtype, phone_number, buf, sizeof(buf));
		gsm_switch_state(gsm, ALLOGSM_STATE_PHONE_CHECK, buf);
	}
	return 0;
}
#endif

